# (c) 2018, NetApp, Inc
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

''' unit test for ONTAP FlexCache Ansible module '''

from __future__ import print_function
import json
import pytest

from units.compat import unittest
from units.compat.mock import patch
from ansible.module_utils import basic
from ansible.module_utils._text import to_bytes
import ansible.module_utils.netapp as netapp_utils

from ansible.modules.storage.netapp.na_ontap_flexcache \
    import NetAppONTAPFlexCache as my_module  # module under test

if not netapp_utils.has_netapp_lib():
    pytestmark = pytest.mark.skip('skipping as missing required netapp_lib')


def set_module_args(args):
    """prepare arguments so that they will be picked up during module creation"""
    args = json.dumps({'ANSIBLE_MODULE_ARGS': args})
    basic._ANSIBLE_ARGS = to_bytes(args)  # pylint: disable=protected-access


class AnsibleExitJson(Exception):
    """Exception class to be raised by module.exit_json and caught by the test case"""
    pass


class AnsibleFailJson(Exception):
    """Exception class to be raised by module.fail_json and caught by the test case"""
    pass


def exit_json(*args, **kwargs):  # pylint: disable=unused-argument
    """function to patch over exit_json; package return data into an exception"""
    if 'changed' not in kwargs:
        kwargs['changed'] = False
    raise AnsibleExitJson(kwargs)


def fail_json(*args, **kwargs):  # pylint: disable=unused-argument
    """function to patch over fail_json; package return data into an exception"""
    kwargs['failed'] = True
    raise AnsibleFailJson(kwargs)


class MockONTAPConnection(object):
    ''' mock server connection to ONTAP host '''

    def __init__(self, kind=None, parm1=None, api_error=None, job_error=None):
        ''' save arguments '''
        self.type = kind
        self.parm1 = parm1
        self.api_error = api_error
        self.job_error = job_error
        self.xml_in = None
        self.xml_out = None

    def invoke_successfully(self, xml, enable_tunneling):  # pylint: disable=unused-argument
        ''' mock invoke_successfully returning xml data '''
        self.xml_in = xml
        tag = xml.get_name()
        if tag == 'flexcache-get-iter' and self.type == 'vserver':
            xml = self.build_flexcache_info(self.parm1)
        elif tag == 'flexcache-create-async':
            xml = self.build_flexcache_create_destroy_rsp()
        elif tag == 'flexcache-destroy-async':
            if self.api_error:
                code, message = self.api_error.split(':', 2)
                raise netapp_utils.zapi.NaApiError(code, message)
            xml = self.build_flexcache_create_destroy_rsp()
        elif tag == 'job-get':
            xml = self.build_job_info(self.job_error)
        self.xml_out = xml
        return xml

    @staticmethod
    def build_flexcache_info(vserver):
        ''' build xml data for vserser-info '''
        xml = netapp_utils.zapi.NaElement('xml')
        attributes = netapp_utils.zapi.NaElement('attributes-list')
        count = 2 if vserver == 'repeats' else 1
        for dummy in range(count):
            attributes.add_node_with_children('flexcache-info', **{
                'vserver': vserver,
                'origin-vserver': 'ovserver',
                'origin-volume': 'ovolume',
                'origin-cluster': 'ocluster',
                'volume': 'volume',
            })
        xml.add_child_elem(attributes)
        xml.add_new_child('num-records', str(count))
        return xml

    @staticmethod
    def build_flexcache_create_destroy_rsp():
        ''' build xml data for a create or destroy response '''
        xml = netapp_utils.zapi.NaElement('xml')
        xml.add_new_child('result-status', 'in_progress')
        xml.add_new_child('result-jobid', '1234')
        return xml

    @staticmethod
    def build_job_info(error):
        ''' build xml data for a job '''
        xml = netapp_utils.zapi.NaElement('xml')
        attributes = netapp_utils.zapi.NaElement('attributes')
        if error is None:
            state = 'success'
        elif error == 'time_out':
            state = 'running'
        else:
            state = 'failure'
        attributes.add_node_with_children('job-info', **{
            'job-state': state,
            'job-progress': 'dummy',
            'job-completion': error,
        })
        xml.add_child_elem(attributes)
        xml.add_new_child('result-status', 'in_progress')
        xml.add_new_child('result-jobid', '1234')
        return xml


class TestMyModule(unittest.TestCase):
    ''' a group of related Unit Tests '''

    def setUp(self):
        self.mock_module_helper = patch.multiple(basic.AnsibleModule,
                                                 exit_json=exit_json,
                                                 fail_json=fail_json)
        self.mock_module_helper.start()
        self.addCleanup(self.mock_module_helper.stop)
        # make sure to change this to False before submitting
        self.onbox = False
        self.dummy_args = dict()
        for arg in ('hostname', 'username', 'password'):
            self.dummy_args[arg] = arg
        if self.onbox:
            self.args = {
                'hostname': '10.193.78.219',
                'username': 'admin',
                'password': 'netapp1!',
            }
        else:
            self.args = self.dummy_args
        self.server = MockONTAPConnection()

    def create_flexcache(self, vserver, volume, junction_path):
        ''' create flexcache '''
        if not self.onbox:
            return
        args = {
            'state': 'present',
            'volume': volume,
            'size': '90',       # 80MB minimum
            'size_unit': 'mb',  # 80MB minimum
            'vserver': vserver,
            'aggr_list': 'aggr1',
            'origin_volume': 'fc_vol_origin',
            'origin_vserver': 'ansibleSVM',
            'junction_path': junction_path,
        }
        args.update(self.args)
        set_module_args(args)
        my_obj = my_module()
        try:
            my_obj.apply()
        except AnsibleExitJson as exc:
            print('Create util: ' + repr(exc))
        except AnsibleFailJson as exc:
            print('Create util: ' + repr(exc))

    def delete_flexcache(self, vserver, volume):
        ''' delete flexcache '''
        if not self.onbox:
            return
        args = {'volume': volume, 'vserver': vserver, 'state': 'absent', 'force_offline': 'true'}
        args.update(self.args)
        set_module_args(args)
        my_obj = my_module()
        try:
            my_obj.apply()
        except AnsibleExitJson as exc:
            print('Delete util: ' + repr(exc))
        except AnsibleFailJson as exc:
            print('Delete util: ' + repr(exc))

    def test_module_fail_when_required_args_missing(self):
        ''' required arguments are reported as errors '''
        with pytest.raises(AnsibleFailJson) as exc:
            set_module_args({})
            my_module()
        print('Info: %s' % exc.value.args[0]['msg'])

    def test_missing_parameters(self):
        ''' fail if origin volume and origin verser are missing '''
        args = {
            'vserver': 'vserver',
            'volume': 'volume'
        }
        args.update(self.dummy_args)
        set_module_args(args)
        my_obj = my_module()
        my_obj.server = self.server
        with pytest.raises(AnsibleFailJson) as exc:
            # It may not be a good idea to start with apply
            # More atomic methods can be easier to mock
            # Hint: start with get methods, as they are called first
            my_obj.apply()
        msg = 'Missing parameters: origin_volume, origin_vserver'
        assert exc.value.args[0]['msg'] == msg

    def test_missing_parameter(self):
        ''' fail if origin verser parameter is missing '''
        args = {
            'vserver': 'vserver',
            'origin_volume': 'origin_volume',
            'volume': 'volume'
        }
        args.update(self.dummy_args)
        set_module_args(args)
        my_obj = my_module()
        my_obj.server = self.server
        with pytest.raises(AnsibleFailJson) as exc:
            my_obj.apply()
        msg = 'Missing parameter: origin_vserver'
        assert exc.value.args[0]['msg'] == msg

    def test_get_flexcache(self):
        ''' get flexcache info '''
        args = {
            'vserver': 'ansibleSVM',
            'origin_volume': 'origin_volume',
            'volume': 'volume'
        }
        args.update(self.args)
        set_module_args(args)
        my_obj = my_module()
        if not self.onbox:
            my_obj.server = MockONTAPConnection('vserver')
        info = my_obj.flexcache_get()
        print('info: ' + repr(info))

    def test_get_flexcache_double(self):
        ''' get flexcache info returns 2 entries! '''
        args = {
            'vserver': 'ansibleSVM',
            'origin_volume': 'origin_volume',
            'volume': 'volume'
        }
        args.update(self.dummy_args)
        set_module_args(args)
        my_obj = my_module()
        my_obj.server = MockONTAPConnection('vserver', 'repeats')
        with pytest.raises(AnsibleFailJson) as exc:
            my_obj.flexcache_get()
        msg = 'Error fetching FlexCache info: Multiple records found for %s:' % args['volume']
        assert exc.value.args[0]['msg'] == msg

    def test_create_flexcache(self):
        ''' create flexcache '''
        args = {
            'volume': 'volume',
            'size': '90',       # 80MB minimum
            'size_unit': 'mb',  # 80MB minimum
            'vserver': 'ansibleSVM',
            'aggr_list': 'aggr1',
            'origin_volume': 'fc_vol_origin',
            'origin_vserver': 'ansibleSVM',
        }
        self.delete_flexcache(args['vserver'], args['volume'])
        args.update(self.args)
        set_module_args(args)
        my_obj = my_module()
        if not self.onbox:
            my_obj.server = MockONTAPConnection()
        with patch.object(my_module, 'flexcache_create', wraps=my_obj.flexcache_create) as mock_create:
            # with patch('__main__.my_module.flexcache_create', wraps=my_obj.flexcache_create) as mock_create:
            with pytest.raises(AnsibleExitJson) as exc:
                my_obj.apply()
            print('Create: ' + repr(exc.value))
            assert exc.value.args[0]['changed']
            mock_create.assert_called_with()

    def test_create_flexcache_idempotent(self):
        ''' create flexcache - already exists '''
        args = {
            'volume': 'volume',
            'vserver': 'ansibleSVM',
            'aggr_list': 'aggr1',
            'origin_volume': 'fc_vol_origin',
            'origin_vserver': 'ansibleSVM',
        }
        args.update(self.args)
        set_module_args(args)
        my_obj = my_module()
        if not self.onbox:
            my_obj.server = MockONTAPConnection('vserver')
        with pytest.raises(AnsibleExitJson) as exc:
            my_obj.apply()
        print('Create: ' + repr(exc.value))
        assert exc.value.args[0]['changed'] is False

    def test_create_flexcache_autoprovision(self):
        ''' create flexcache with autoprovision'''
        args = {
            'volume': 'volume',
            'size': '90',       # 80MB minimum
            'size_unit': 'mb',  # 80MB minimum
            'vserver': 'ansibleSVM',
            'auto_provision_as': 'flexgroup',
            'origin_volume': 'fc_vol_origin',
            'origin_vserver': 'ansibleSVM',
        }
        self.delete_flexcache(args['vserver'], args['volume'])
        args.update(self.args)
        set_module_args(args)
        my_obj = my_module()
        if not self.onbox:
            my_obj.server = MockONTAPConnection()
        with patch.object(my_module, 'flexcache_create', wraps=my_obj.flexcache_create) as mock_create:
            with pytest.raises(AnsibleExitJson) as exc:
                my_obj.apply()
            print('Create: ' + repr(exc.value))
            assert exc.value.args[0]['changed']
            mock_create.assert_called_with()

    def test_create_flexcache_autoprovision_idempotent(self):
        ''' create flexcache with autoprovision - already exists '''
        args = {
            'volume': 'volume',
            'vserver': 'ansibleSVM',
            'origin_volume': 'fc_vol_origin',
            'origin_vserver': 'ansibleSVM',
            'auto_provision_as': 'flexgroup',
        }
        args.update(self.args)
        set_module_args(args)
        my_obj = my_module()
        if not self.onbox:
            my_obj.server = MockONTAPConnection('vserver')
        with pytest.raises(AnsibleExitJson) as exc:
            my_obj.apply()
        print('Create: ' + repr(exc.value))
        assert exc.value.args[0]['changed'] is False

    def test_create_flexcache_multiplier(self):
        ''' create flexcache with aggregate multiplier'''
        args = {
            'volume': 'volume',
            'size': '90',       # 80MB minimum
            'size_unit': 'mb',  # 80MB minimum
            'vserver': 'ansibleSVM',
            'aggr_list': 'aggr1',
            'origin_volume': 'fc_vol_origin',
            'origin_vserver': 'ansibleSVM',
            'aggr_list_multiplier': '2',
        }
        self.delete_flexcache(args['vserver'], args['volume'])
        args.update(self.args)
        set_module_args(args)
        my_obj = my_module()
        if not self.onbox:
            my_obj.server = MockONTAPConnection()
        with patch.object(my_module, 'flexcache_create', wraps=my_obj.flexcache_create) as mock_create:
            with pytest.raises(AnsibleExitJson) as exc:
                my_obj.apply()
            print('Create: ' + repr(exc.value))
            assert exc.value.args[0]['changed']
            mock_create.assert_called_with()

    def test_create_flexcache_multiplier_idempotent(self):
        ''' create flexcache with aggregate multiplier - already exists '''
        args = {
            'volume': 'volume',
            'vserver': 'ansibleSVM',
            'aggr_list': 'aggr1',
            'origin_volume': 'fc_vol_origin',
            'origin_vserver': 'ansibleSVM',
            'aggr_list_multiplier': '2',
        }
        args.update(self.args)
        set_module_args(args)
        my_obj = my_module()
        if not self.onbox:
            my_obj.server = MockONTAPConnection('vserver')
        with pytest.raises(AnsibleExitJson) as exc:
            my_obj.apply()
        print('Create: ' + repr(exc.value))
        assert exc.value.args[0]['changed'] is False

    def test_delete_flexcache_exists_no_force(self):
        ''' delete flexcache '''
        args = {'volume': 'volume', 'vserver': 'ansibleSVM', 'state': 'absent'}
        args.update(self.args)
        set_module_args(args)
        my_obj = my_module()
        error = '13001:Volume volume in Vserver ansibleSVM must be offline to be deleted. ' \
                'Use "volume offline -vserver ansibleSVM -volume volume" command to offline ' \
                'the volume'
        if not self.onbox:
            my_obj.server = MockONTAPConnection('vserver', 'flex', api_error=error)
        with patch.object(my_module, 'flexcache_delete', wraps=my_obj.flexcache_delete) as mock_delete:
            with pytest.raises(AnsibleFailJson) as exc:
                my_obj.apply()
            print('Delete: ' + repr(exc.value))
            msg = 'Error deleting FlexCache : NetApp API failed. Reason - %s' % error
            assert exc.value.args[0]['msg'] == msg
            mock_delete.assert_called_with()

    def test_delete_flexcache_exists_with_force(self):
        ''' delete flexcache '''
        args = {'volume': 'volume', 'vserver': 'ansibleSVM', 'state': 'absent', 'force_offline': 'true'}
        args.update(self.args)
        set_module_args(args)
        my_obj = my_module()
        if not self.onbox:
            my_obj.server = MockONTAPConnection('vserver', 'flex')
        with patch.object(my_module, 'flexcache_delete', wraps=my_obj.flexcache_delete) as mock_delete:
            with pytest.raises(AnsibleExitJson) as exc:
                my_obj.apply()
            print('Delete: ' + repr(exc.value))
            assert exc.value.args[0]['changed']
            mock_delete.assert_called_with()

    def test_delete_flexcache_exists_junctionpath_no_force(self):
        ''' delete flexcache '''
        args = {'volume': 'volume', 'vserver': 'ansibleSVM', 'junction_path': 'jpath', 'state': 'absent', 'force_offline': 'true'}
        self.create_flexcache(args['vserver'], args['volume'], args['junction_path'])
        args.update(self.args)
        set_module_args(args)
        my_obj = my_module()
        error = '160:Volume volume on Vserver ansibleSVM must be unmounted before being taken offline or restricted.'
        if not self.onbox:
            my_obj.server = MockONTAPConnection('vserver', 'flex', api_error=error)
        with patch.object(my_module, 'flexcache_delete', wraps=my_obj.flexcache_delete) as mock_delete:
            with pytest.raises(AnsibleFailJson) as exc:
                my_obj.apply()
            print('Delete: ' + repr(exc.value))
            msg = 'Error deleting FlexCache : NetApp API failed. Reason - %s' % error
            assert exc.value.args[0]['msg'] == msg
            mock_delete.assert_called_with()

    def test_delete_flexcache_exists_junctionpath_with_force(self):
        ''' delete flexcache '''
        args = {'volume': 'volume', 'vserver': 'ansibleSVM', 'junction_path': 'jpath', 'state': 'absent', 'force_offline': 'true', 'force_unmount': 'true'}
        self.create_flexcache(args['vserver'], args['volume'], args['junction_path'])
        args.update(self.args)
        set_module_args(args)
        my_obj = my_module()
        if not self.onbox:
            my_obj.server = MockONTAPConnection('vserver', 'flex')
        with patch.object(my_module, 'flexcache_delete', wraps=my_obj.flexcache_delete) as mock_delete:
            with pytest.raises(AnsibleExitJson) as exc:
                my_obj.apply()
            print('Delete: ' + repr(exc.value))
            assert exc.value.args[0]['changed']
            mock_delete.assert_called_with()

    def test_delete_flexcache_not_exist(self):
        ''' delete flexcache '''
        args = {'volume': 'volume', 'vserver': 'ansibleSVM', 'state': 'absent'}
        args.update(self.args)
        set_module_args(args)
        my_obj = my_module()
        if not self.onbox:
            my_obj.server = MockONTAPConnection()
        with pytest.raises(AnsibleExitJson) as exc:
            my_obj.apply()
        print('Delete: ' + repr(exc.value))
        assert exc.value.args[0]['changed'] is False

    def test_create_flexcache_size_error(self):
        ''' create flexcache '''
        args = {
            'volume': 'volume_err',
            'size': '50',       # 80MB minimum
            'size_unit': 'mb',  # 80MB minimum
            'vserver': 'ansibleSVM',
            'aggr_list': 'aggr1',
            'origin_volume': 'fc_vol_origin',
            'origin_vserver': 'ansibleSVM',
        }
        args.update(self.args)
        set_module_args(args)
        my_obj = my_module()
        error = 'Size "50MB" ("52428800B") is too small.  Minimum size is "80MB" ("83886080B"). '
        if not self.onbox:
            my_obj.server = MockONTAPConnection(job_error=error)
        with patch.object(my_module, 'flexcache_create', wraps=my_obj.flexcache_create) as mock_create:
            with pytest.raises(AnsibleFailJson) as exc:
                my_obj.apply()
            print('Create: ' + repr(exc.value))
            msg = 'Error when creating flexcache: %s' % error
            assert exc.value.args[0]['msg'] == msg
            mock_create.assert_called_with()

    def test_create_flexcache_time_out(self):
        ''' create flexcache '''
        args = {
            'volume': 'volume_err',
            'size': '50',       # 80MB minimum
            'size_unit': 'mb',  # 80MB minimum
            'vserver': 'ansibleSVM',
            'aggr_list': 'aggr1',
            'origin_volume': 'fc_vol_origin',
            'origin_vserver': 'ansibleSVM',
            'time_out': '2'
        }
        args.update(self.args)
        set_module_args(args)
        my_obj = my_module()
        if not self.onbox:
            my_obj.server = MockONTAPConnection(job_error='time_out')
        with patch.object(my_module, 'flexcache_create', wraps=my_obj.flexcache_create) as mock_create:
            with pytest.raises(AnsibleFailJson) as exc:
                my_obj.apply()
            print('Create: ' + repr(exc.value))
            msg = 'Error when creating flexcache: job completion exceeded expected timer of: %s seconds' \
                % args['time_out']
            assert exc.value.args[0]['msg'] == msg
            mock_create.assert_called_with()



#!/usr/bin/python

# Copyright: (c) 2018, Juergen Wiebe <wiebe@e-spirit.com>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {
    'metadata_version': '1.1',
    'status': ['preview'],
    'supported_by': 'community'
}

DOCUMENTATION = """
---
module: utm_network_interface_address

author:
    - Juergen Wiebe (@steamx)

short_description: Create, update or destroy network/interface_address object

description:
    - Create, update or destroy a network/interface_address object in SOPHOS UTM.
    - This module needs to have the REST Ability of the UTM to be activated.

version_added: "2.8"

options:
    name:
        description:
          - The name of the object. Will be used to identify the entry
        required: true
    address:
        description:
          - The ip4 address of the network/interface_address object.
        required: true
    address6:
        description:
          - The ip6 address of the network/interface_address object.
        required: false
    comment:
        description:
          - An optional comment to add to the object
    resolved:
        description:
          - Whether or not the object is resolved
    resolved6:
        description:
          - Whether or not the object is resolved

extends_documentation_fragment:
    - utm
"""

EXAMPLES = """
# Create a network interface address
- name: utm network interface address
  utm_proxy_backend:
    utm_host: sophos.host.name
    utm_token: abcdefghijklmno1234
    name: TestNetworkInterfaceAddress
    address: 0.0.0.0
    state: present

# Remove a network interface address
- name: utm network interface address
  network_interface_address:
    utm_host: sophos.host.name
    utm_token: abcdefghijklmno1234
    name: TestNetworkInterfaceAddress
    address: 0.0.0.0
    state: absent
"""

RETURN = """
result:
    description: The utm object that was created
    returned: success
    type: complex
    contains:
        _ref:
            description: The reference name of the object
            type: string
        _locked:
            description: Whether or not the object is currently locked
            type: boolean
        _type:
            description: The type of the object
            type: string
        name:
            description: The name of the object
            type: string
        address:
             description: The ip4 address of the network/interface_address object
             type: string
        address6:
             description: The ip6 address of the network/interface_address object
             type: string
        comment:
            description: The comment string
            type: string
        resolved:
             description: Whether or not the object is resolved
             type: boolean
        resolved6:
             description: Whether or not the object is resolved
             type: boolean
"""

from ansible.module_utils.utm_utils import UTM, UTMModule
from ansible.module_utils._text import to_native


def main():
    endpoint = "network/interface_address"
    key_to_check_for_changes = ["comment", "address"]
    module = UTMModule(
        argument_spec=dict(
            name=dict(type='str', required=True),
            address=dict(type='str', required=True),
            comment=dict(type='str', required=False, default=""),
            address6=dict(type='str', required=False),
            resolved=dict(type='boolean', required=False),
            resolved6=dict(type='boolean', required=False)
        )
    )
    try:
        UTM(module, endpoint, key_to_check_for_changes).execute()
    except Exception as e:
        module.fail_json(msg=to_native(e))


if __name__ == '__main__':
    main()



import sys

import units.compat.unittest as unittest
from units.compat.mock import MagicMock
from units.compat.unittest import TestCase
from units.modules.utils import set_module_args

# Exoscale's cs doesn't support Python 2.6
if sys.version_info[:2] != (2, 6):
    from ansible.modules.cloud.cloudstack.cs_traffic_type import AnsibleCloudStackTrafficType, setup_module_object


EXISTING_TRAFFIC_TYPES_RESPONSE = {
    "count": 3,
    "traffictype": [
        {
            "id": "9801cf73-5a73-4883-97e4-fa20c129226f",
            "kvmnetworklabel": "cloudbr0",
            "physicalnetworkid": "659c1840-9374-440d-a412-55ca360c9d3c",
            "traffictype": "Management"
        },
        {
            "id": "28ed70b7-9a1f-41bf-94c3-53a9f22da8b6",
            "kvmnetworklabel": "cloudbr0",
            "physicalnetworkid": "659c1840-9374-440d-a412-55ca360c9d3c",
            "traffictype": "Guest"
        },
        {
            "id": "9c05c802-84c0-4eda-8f0a-f681364ffb46",
            "kvmnetworklabel": "cloudbr0",
            "physicalnetworkid": "659c1840-9374-440d-a412-55ca360c9d3c",
            "traffictype": "Storage"
        }
    ]
}

VALID_LIST_NETWORKS_RESPONSE = {
    "count": 1,
    "physicalnetwork": [
        {
            "broadcastdomainrange": "ZONE",
            "id": "659c1840-9374-440d-a412-55ca360c9d3c",
            "name": "eth1",
            "state": "Enabled",
            "vlan": "3900-4000",
            "zoneid": "49acf813-a8dd-4da0-aa53-1d826d6003e7"
        }
    ]
}

VALID_LIST_ZONES_RESPONSE = {
    "count": 1,
    "zone": [
        {
            "allocationstate": "Enabled",
            "dhcpprovider": "VirtualRouter",
            "dns1": "8.8.8.8",
            "dns2": "8.8.4.4",
            "guestcidraddress": "10.10.0.0/16",
            "id": "49acf813-a8dd-4da0-aa53-1d826d6003e7",
            "internaldns1": "192.168.56.1",
            "localstorageenabled": True,
            "name": "DevCloud-01",
            "networktype": "Advanced",
            "securitygroupsenabled": False,
            "tags": [],
            "zonetoken": "df20d65a-c6c8-3880-9064-4f77de2291ef"
        }
    ]
}


base_module_args = {
    "api_key": "api_key",
    "api_secret": "very_secret_content",
    "api_url": "http://localhost:8888/api/client",
    "kvm_networklabel": "cloudbr0",
    "physical_network": "eth1",
    "poll_async": True,
    "state": "present",
    "traffic_type": "Guest",
    "zone": "DevCloud-01"
}


class TestAnsibleCloudstackTraffiType(TestCase):

    @unittest.skipUnless(sys.version_info[:2] >= (2, 7), "Exoscale's cs doesn't support Python 2.6")
    def test_module_is_created_sensibly(self):
        set_module_args(base_module_args)
        module = setup_module_object()
        assert module.params['traffic_type'] == 'Guest'

    @unittest.skipUnless(sys.version_info[:2] >= (2, 7), "Exoscale's cs doesn't support Python 2.6")
    def test_update_called_when_traffic_type_exists(self):
        set_module_args(base_module_args)
        module = setup_module_object()
        actt = AnsibleCloudStackTrafficType(module)
        actt.get_traffic_type = MagicMock(return_value=EXISTING_TRAFFIC_TYPES_RESPONSE['traffictype'][0])
        actt.update_traffic_type = MagicMock()
        actt.present_traffic_type()
        self.assertTrue(actt.update_traffic_type.called)

    @unittest.skipUnless(sys.version_info[:2] >= (2, 7), "Exoscale's cs doesn't support Python 2.6")
    def test_update_not_called_when_traffic_type_doesnt_exist(self):
        set_module_args(base_module_args)
        module = setup_module_object()
        actt = AnsibleCloudStackTrafficType(module)
        actt.get_traffic_type = MagicMock(return_value=None)
        actt.update_traffic_type = MagicMock()
        actt.add_traffic_type = MagicMock()
        actt.present_traffic_type()
        self.assertFalse(actt.update_traffic_type.called)
        self.assertTrue(actt.add_traffic_type.called)

    @unittest.skipUnless(sys.version_info[:2] >= (2, 7), "Exoscale's cs doesn't support Python 2.6")
    def test_traffic_type_returned_if_exists(self):
        set_module_args(base_module_args)
        module = setup_module_object()
        actt = AnsibleCloudStackTrafficType(module)
        actt.get_physical_network = MagicMock(return_value=VALID_LIST_NETWORKS_RESPONSE['physicalnetwork'][0])
        actt.get_traffic_types = MagicMock(return_value=EXISTING_TRAFFIC_TYPES_RESPONSE)
        tt = actt.present_traffic_type()
        self.assertTrue(tt.get('kvmnetworklabel') == base_module_args['kvm_networklabel'])
        self.assertTrue(tt.get('traffictype') == base_module_args['traffic_type'])


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'community'}

DOCUMENTATION = r'''
---
module: intersight_rest_api
short_description: REST API configuration for Cisco Intersight
description:
- Direct REST API configuration for Cisco Intersight.
- All REST API resources and properties must be specified.
- For more information see L(Cisco Intersight,https://intersight.com/apidocs).
extends_documentation_fragment: intersight
options:
  resource_path:
    description:
    - Resource URI being configured related to api_uri.
    type: str
    required: yes
  query_params:
    description:
    - Query parameters for the Intersight API query languange.
    type: dict
  update_method:
    description:
    - The HTTP method used for update operations.
    - Some Intersight resources require POST operations for modifications.
    type: str
    choices: [ patch, post ]
    default: patch
  api_body:
    description:
    - The paylod for API requests used to modify resources.
    type: dict
  state:
    description:
    - If C(present), will verify the resource is present and will create if needed.
    - If C(absent), will verify the resource is absent and will delete if needed.
    choices: [present, absent]
    default: present
author:
- David Soper (@dsoper2)
- CiscoUcs (@CiscoUcs)
version_added: '2.8'
'''

EXAMPLES = r'''
- name: Configure Boot Policy
  intersight_rest_api:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    api_key_uri: "{{ api_key_uri }}"
    validate_certs: "{{ validate_certs }}"
    resource_path: /boot/PrecisionPolicies
    query_params:
      $filter: "Name eq 'vmedia-localdisk'"
    api_body: {
      "Name": "vmedia-hdd",
      "ConfiguredBootMode": "Legacy",
      "BootDevices": [
        {
          "ObjectType": "boot.VirtualMedia",
          "Enabled": true,
          "Name": "remote-vmedia",
          "Subtype": "cimc-mapped-dvd"
        },
        {
          "ObjectType": "boot.LocalDisk",
          "Enabled": true,
          "Name": "localdisk",
          "Slot": "MRAID",
          "Bootloader": null
        }
      ],
    }
    state: present

- name: Delete Boot Policy
  intersight_rest_api:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    api_key_uri: "{{ api_key_uri }}"
    validate_certs: "{{ validate_certs }}"
    resource_path: /boot/PrecisionPolicies
    query_params:
      $filter: "Name eq 'vmedia-localdisk'"
    state: absent
'''

RETURN = r'''
api_repsonse:
  description: The API response output returned by the specified resource.
  returned: always
  type: dict
  sample:
    "api_response": {
      "BootDevices": [
        {
          "Enabled": true,
          "Name": "remote-vmedia",
          "ObjectType": "boot.VirtualMedia",
          "Subtype": "cimc-mapped-dvd"
        },
        {
          "Bootloader": null,
          "Enabled": true,
          "Name": "boot-lun",
          "ObjectType": "boot.LocalDisk",
          "Slot": "MRAID"
        }
      ],
      "ConfiguredBootMode": "Legacy",
      "Name": "vmedia-localdisk",
      "ObjectType": "boot.PrecisionPolicy",
    }
'''


import re
from ansible.module_utils.remote_management.intersight import IntersightModule, intersight_argument_spec
from ansible.module_utils.basic import AnsibleModule
from ansible.module_utils.six import iteritems


def get_resource(intersight):
    '''
    GET a resource and return the 1st element found
    '''
    options = {
        'http_method': 'get',
        'resource_path': intersight.module.params['resource_path'],
        'query_params': intersight.module.params['query_params'],
    }
    response_dict = intersight.call_api(**options)
    if response_dict.get('Results'):
        # return the 1st list element
        response_dict = response_dict['Results'][0]

    return response_dict


def compare_values(expected, actual):
    try:
        for (key, value) in iteritems(expected):
            if re.search(r'P(ass)?w(or)?d', key) or key not in actual:
                # do not compare any password related attributes or attributes that are not in the actual resource
                continue
            if not compare_values(value, actual[key]):
                return False
        # loop complete with all items matching
        return True
    except (AttributeError, TypeError):
        if expected and actual != expected:
            return False
        return True


def configure_resource(intersight, moid):
    if not intersight.module.check_mode:
        if moid:
            # update the resource - user has to specify all the props they want updated
            options = {
                'http_method': intersight.module.params['update_method'],
                'resource_path': intersight.module.params['resource_path'],
                'body': intersight.module.params['api_body'],
                'moid': moid,
            }
            response_dict = intersight.call_api(**options)
            if response_dict.get('Results'):
                # return the 1st element in the results list
                intersight.result['api_response'] = response_dict['Results'][0]
        else:
            # create the resource
            options = {
                'http_method': 'post',
                'resource_path': intersight.module.params['resource_path'],
                'body': intersight.module.params['api_body'],
            }
            resp = intersight.call_api(**options)
            if 'Moid' not in resp:
                resp = get_resource(intersight)
            intersight.result['api_response'] = resp
    intersight.result['changed'] = True


def delete_resource(intersight, moid):
    # delete resource and create empty api_response
    if not intersight.module.check_mode:
        options = {
            'http_method': 'delete',
            'resource_path': intersight.module.params['resource_path'],
            'moid': moid,
        }
        intersight.call_api(**options)
        intersight.result['api_response'] = {}
    intersight.result['changed'] = True


def main():
    argument_spec = intersight_argument_spec
    argument_spec.update(
        resource_path=dict(type='str', required=True),
        query_params=dict(type='dict', default={}),
        update_method=dict(type='str', choices=['patch', 'post'], default='patch'),
        api_body=dict(type='dict', default={}),
        state=dict(type='str', choices=['absent', 'present'], default='present'),
    )

    module = AnsibleModule(
        argument_spec,
        supports_check_mode=True,
    )

    intersight = IntersightModule(module)
    intersight.result['api_response'] = {}

    # get the current state of the resource
    intersight.result['api_response'] = get_resource(intersight)

    # determine requested operation (config, delete, or neither (get resource only))
    if module.params['state'] == 'present':
        request_delete = False
        # api_body implies resource configuration through post/patch
        request_config = bool(module.params['api_body'])
    else:  # state == 'absent'
        request_delete = True
        request_config = False

    moid = None
    resource_values_match = False
    if (request_config or request_delete) and intersight.result['api_response'].get('Moid'):
        # resource exists and moid was returned
        moid = intersight.result['api_response']['Moid']
        if request_config:
            resource_values_match = compare_values(module.params['api_body'], intersight.result['api_response'])
        else:  # request_delete
            delete_resource(intersight, moid)

    if request_config and not resource_values_match:
        configure_resource(intersight, moid)

    module.exit_json(**intersight.result)


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# (c) 2019, Ansible by Red Hat, inc
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'community'}

DOCUMENTATION = """
---
module: asa_og
version_added: "2.8"
author:
    - "Federico Olivieri (@Federico87)"
short_description: Manage object groups on a Cisco ASA
description:
    - This module allows you to create and update object-group network/service on Cisco ASA device.
options:
    name:
        description:
            - Name of the object group.
        required: true
    group_type:
        description:
            - The object group type.
        choices: ['network-object', 'service-object', 'port-object']
        required: true
    protocol:
        description:
            - The protocol for object-group service with port-object.
        choices: ['udp', 'tcp', 'tcp-udp']
    host_ip:
        description:
            - The host IP address for object-group network.
        type: list
    description:
        description:
            - The description for the object-group.
    group_object:
        description:
            - The group-object for network object-group.
        type: list
    ip_mask:
        description:
            - The IP address and mask for network object-group.
        type: list
    port_range:
        description:
            - The port range for port-object.
    port_eq:
        description:
            - The single port for port-object.
    service_cfg:
        description:
            - The service-object configuration protocol, direction, range or port.
    state:
        description:
            - Manage the state of the resource.
        default: present
        choices: ['present', 'absent', 'replace']
"""

EXAMPLES = """
---
- name: configure network object-group
  asa_og:
    name: ansible_test_0
    group_type: network-object
    state: present
    description: ansible_test object-group description
    host_ip:
      - 8.8.8.8
      - 8.8.4.4
    ip_mask:
      - 10.0.0.0 255.255.255.0
      - 192.168.0.0 255.255.0.0
    group_object:
      - awx_lon
      - awx_ams

- name: configure port-object object-group
  asa_og:
    name: ansible_test_1
    group_type: port-object
    state: replace
    description: ansible_test object-group description
    protocol: tcp-udp
    port_eq:
      - 1025
      - kerberos
    port_range:
      - 1025 5201
      - 0 1024

- name: configure service-object object-group
  asa_og:
    name: ansible_test_2
    group_type: service-object
    state: absent
    description: ansible_test object-group description
    service_cfg:
      - tcp destination eq 8080
      - tcp destination eq www
"""

RETURN = """
commands:
  description: command sent to the device
  returned: always
  type: list
  sample: [
    "object-group network ansible_test_0",
    "description ansible_test object-group description",
    "network-object host 8.8.8.8",
    "network-object host 8.8.4.4",
    "network-object 10.0.0.0 255.255.255.0",
    "network-object 192.168.0.0 255.255.0.0",
    "network-object 192.168.0.0 255.255.0.0",
    "group-object awx_lon",
    "group-object awx_ams",
    ]
"""
import re
import sys

from ansible.module_utils.basic import AnsibleModule
from ansible.module_utils.network.asa.asa import check_args
from ansible.module_utils.network.asa.asa import get_config, load_config, run_commands
from ansible.module_utils.network.common.config import NetworkConfig, dumps


class Parser():
    '''Regex class for outputs parsing'''

    def __init__(self, config, protocol):
        '''Parser __init__ method'''
        self.config = config
        self.protocol = protocol

    def parse_obj_grp_name(self):
        list_return = list()
        match = re.search(r'(?:object-group\s)(network\s|service\s)(\w+)\s?(tcp-udp|tcp|udp)?', self.config, re.M)

        if match:
            if match.group(3):
                list_return.append(str(match.group(3)))
            else:
                list_return.append(False)

            if match.group(2):
                list_return.append(str(match.group(2)))

            if match.group(1):
                list_return.append(str(match.group(1)))

        return list_return

    def parse_description(self):
        match = re.search(r'(description\s)(.*)', self.config, re.M)
        if match:
            description = match.group(2)

            return description

    def parse_host(self):
        list_return = list()
        match = re.findall(r'(host\s)(\d+\.\d+\.\d+\.\d+)', self.config, re.M)

        if match:
            for i in match:
                if i[1]:
                    list_return.append(str(i[1]))

        return list_return

    def parse_group_object(self):
        list_return = list()
        match = re.findall(r'(group-object\s)(.*)', self.config, re.M)

        if match:
            for i in match:
                if i[1]:
                    list_return.append(str(i[1]))

        return list_return

    def parse_address(self):
        list_return = list()
        match = re.findall(r'(network-object\s)(\d+\.\d+\.\d+\.\d+\s\d+\.\d+\.\d+\.\d+)', self.config, re.M)

        if match:
            for i in match:
                if i[1]:
                    list_return.append(str(i[1]))

        return list_return

    def parse_port_range(self):
        list_return = list()
        match = re.findall(r'(range\s)(.*)', self.config, re.M)

        if match:
            for i in match:
                if i[1]:
                    list_return.append(str(i[1]))

        return list_return

    def parse_port_eq(self):
        list_return = list()
        match = re.findall(r'(eq\s)(.*)', self.config, re.M)

        if match:
            for i in match:
                if i[1]:
                    list_return.append(str(i[1]))

        return list_return

    def parse_service_cfg(self):
        list_return = list()
        match = re.findall(r'(service-object\s)(.*)', self.config, re.M)

        if match:
            for i in match:
                if i[1]:
                    list_return.append(str(i[1]))

        return list_return


def map_config_to_obj(module):

    obj = list()
    obj_dict = dict()

    group_type = module.params['group_type']
    group_name = module.params['name']
    protocol = module.params['protocol']

    sh_run_group_name = get_config(module, flags=['object-group | include {0}'.format(group_name)])
    run_group_name = Parser(sh_run_group_name, protocol).parse_obj_grp_name()

    obj_dict['have_name'] = run_group_name

    if run_group_name:
        if run_group_name[0] is not False:
            obj_dict['have_group_type'] = "port-object"
            obj_dict['have_protocol'] = run_group_name[0]
        elif 'network' in run_group_name[2]:
            obj_dict['have_group_type'] = "network-object"
        elif 'service' in run_group_name[2] and run_group_name[0] is False:
            obj_dict['have_group_type'] = "service-object"
        else:
            obj_dict['have_group_type'] = None

    sh_run_group_type = get_config(module, flags=['object-group id {0}'.format(group_name)])

    have_description = Parser(sh_run_group_type, protocol).parse_description()
    obj_dict['have_description'] = have_description

    have_host_ip = Parser(sh_run_group_type, protocol).parse_host()
    obj_dict['have_host_ip'] = have_host_ip

    have_group_object = Parser(sh_run_group_type, protocol).parse_group_object()
    obj_dict['have_group_object'] = have_group_object

    have_ip_mask = Parser(sh_run_group_type, protocol).parse_address()
    obj_dict['have_ip_mask'] = have_ip_mask

    have_port_range = Parser(sh_run_group_type, protocol).parse_port_range()
    obj_dict['have_port_range'] = have_port_range

    have_port_eq = Parser(sh_run_group_type, protocol).parse_port_eq()
    obj_dict['have_port_eq'] = have_port_eq

    have_service_cfg = Parser(sh_run_group_type, protocol).parse_service_cfg()

    if have_service_cfg:
        have_lines = list()
        for i in have_service_cfg:
            have_lines.append(i.rstrip(' '))
        obj_dict['have_service_cfg'] = have_lines
    elif have_service_cfg is None:
        obj_dict['have_service_cfg'] = have_service_cfg

    obj.append(obj_dict)

    return obj


def replace(want_dict, have):

    commands = list()
    add_lines = list()
    remove_lines = list()

    have_name = have[0].get('have_name')
    have_group_type = have[0].get('have_group_type')
    have_config = have[0].get('have_lines')
    have_description = have[0].get('have_description')
    have_host_ip = have[0].get('have_host_ip')
    have_group_object = have[0].get('have_group_object')
    have_ip_mask = have[0].get('have_ip_mask')
    have_protocol = have[0].get('have_protocol')
    have_port_range = have[0].get('have_port_range')
    have_port_eq = have[0].get('have_port_eq')
    have_service_cfg = have[0].get('have_service_cfg')

    name = want_dict['name']
    group_type = want_dict['group_type']
    protocol = want_dict['protocol']
    description = want_dict['description']
    host = want_dict['host_ip']
    group_object = want_dict['group_object']
    address = want_dict['ip_mask']
    port_range = want_dict['port_range']
    port_eq = want_dict['port_eq']
    service_cfg = want_dict['service_cfg']

    if 'network-object' in group_type:

        if have_group_type is None:
            commands.append('object-group network {0}'.format(name))

            if host:
                for i in host:
                    commands.append('network-object host ' + i)
            if description:
                if have_description is None:
                    commands.append('description {0}'.format(description))
            if group_object:
                for i in group_object:
                    if i not in have_group_object:
                        commands.append('group-object ' + i)
            if address:
                for i in address:
                    commands.append('network-object ' + i)

        elif 'network' in have_group_type:

            if host:
                if sorted(host) != sorted(have_host_ip):
                    for i in host:
                        if i not in have_host_ip:
                            if 'object-group network {0}'.format(name) not in commands:
                                commands.append('object-group network {0}'.format(name))
                            add_lines.append('network-object host ' + i)
                    for i in have_host_ip:
                        if i not in host:
                            if 'object-group network {0}'.format(name) not in commands:
                                commands.append('object-group network {0}'.format(name))
                            remove_lines.append('no network-object host ' + i)

            if description:
                if description != have_description:
                    if 'object-group network {0}'.format(name) not in commands:
                        commands.append('object-group network {0}'.format(name))
                    add_lines.append('description {0}'.format(description))

            if group_object:
                if sorted(group_object) != sorted(have_group_object):
                    for i in group_object:
                        if i not in have_group_object:
                            if 'object-group network {0}'.format(name) not in commands:
                                commands.append('object-group network {0}'.format(name))
                            add_lines.append('group-object ' + i)
                    for i in have_group_object:
                        if i not in group_object:
                            if 'object-group network {0}'.format(name) not in commands:
                                commands.append('object-group network {0}'.format(name))
                            remove_lines.append('no group-object ' + i)
            if address:
                if sorted(address) != sorted(have_ip_mask):
                    for i in address:
                        if i not in have_ip_mask:
                            if 'object-group network {0}'.format(name) not in commands:
                                commands.append('object-group network {0}'.format(name))
                            add_lines.append('network-object ' + i)
                    for i in have_ip_mask:
                        if i not in address:
                            if 'object-group network {0}'.format(name) not in commands:
                                commands.append('object-group network {0}'.format(name))
                            remove_lines.append('no network-object ' + i)

    elif 'port-object' in group_type:

        if have_group_type is None and have_protocol != protocol:
            commands.append('object-group service {0} {1}'.format(name, protocol))

            if port_range:
                for i in port_range:
                    commands.append('port-object range ' + i)
            if port_eq:
                for i in port_eq:
                    commands.append('port-object eq ' + i)
            if description:
                commands.append('description {0}'.format(description))

        elif 'port' in have_group_type and have_protocol == protocol:

            if port_range:
                if sorted(port_range) != sorted(have_port_range):
                    for i in port_range:
                        if i not in have_port_range:
                            if 'object-group service {0} {1}'.format(name, protocol) not in commands:
                                commands.append('object-group service {0} {1}'.format(name, protocol))
                            add_lines.append('port-object range ' + i)
                    for i in have_port_range:
                        if i not in port_range:
                            if 'object-group service {0} {1}'.format(name, protocol) not in commands:
                                commands.append('object-group service {0} {1}'.format(name, protocol))
                            remove_lines.append('no port-object range ' + i)
            if port_eq:
                if sorted(port_eq) != sorted(have_port_eq):
                    for i in port_eq:
                        if i not in have_port_eq:
                            if 'object-group service {0} {1}'.format(name, protocol) not in commands:
                                commands.append('object-group service {0} {1}'.format(name, protocol))
                            add_lines.append('port-object eq ' + i)
                    for i in have_port_eq:
                        if i not in port_eq:
                            if 'object-group service {0} {1}'.format(name, protocol) not in commands:
                                commands.append('object-group service {0} {1}'.format(name, protocol))
                            remove_lines.append('no port-object eq ' + i)
            if description:
                if description != have_description:
                    if 'object-group service {0} {1}'.format(name, protocol) not in commands:
                        commands.append('object-group service {0} {1}'.format(name, protocol))
                    commands.append('description {0}'.format(description))

    elif 'service-object' in group_type:

        if have_group_type is None:
            commands.append('object-group service {0}'.format(name))

            if description:
                if have_description is None:
                    commands.append('description {0}'.format(description))
            if service_cfg:
                for i in service_cfg:
                    commands.append('service-object ' + i)

        elif 'service' in have_group_type:
            if description:
                if description != have_description:
                    if 'object-group service {0}'.format(name) not in commands:
                        commands.append('object-group service {0}'.format(name))
                    commands.append('description {0}'.format(description))
            if service_cfg:
                for i in service_cfg:
                    if i not in have_service_cfg:
                        if 'object-group service {0}'.format(name) not in commands:
                            commands.append('object-group service {0}'.format(name))
                        add_lines.append('service ' + i)
                for i in have_service_cfg:
                    if i not in service_cfg:
                        if 'object-group service {0}'.format(name) not in commands:
                            commands.append('object-group service {0}'.format(name))
                        remove_lines.append('no service ' + i)

    set_add_lines = set(add_lines)
    set_remove_lines = set(remove_lines)

    for i in list(set_add_lines) + list(set_remove_lines):
        commands.append(i)

    return commands


def present(want_dict, have):

    commands = list()

    have_name = have[0].get('have_name')
    have_group_type = have[0].get('have_group_type')
    have_config = have[0].get('have_lines')
    have_description = have[0].get('have_description')
    have_host_ip = have[0].get('have_host_ip')
    have_group_object = have[0].get('have_group_object')
    have_ip_mask = have[0].get('have_ip_mask')
    have_protocol = have[0].get('have_protocol')
    have_port_range = have[0].get('have_port_range')
    have_port_eq = have[0].get('have_port_eq')
    have_service_cfg = have[0].get('have_service_cfg')

    name = want_dict['name']
    group_type = want_dict['group_type']
    protocol = want_dict['protocol']
    description = want_dict['description']
    host = want_dict['host_ip']
    group_object = want_dict['group_object']
    address = want_dict['ip_mask']
    port_range = want_dict['port_range']
    port_eq = want_dict['port_eq']
    service_cfg = want_dict['service_cfg']

    if 'network-object' in group_type:

        if have_group_type is None:
            commands.append('object-group network {0}'.format(name))

            if host:
                for i in host:
                    commands.append('network-object host ' + i)
            if description:
                if have_description is None:
                    commands.append('description {0}'.format(description))
            if group_object:
                for i in group_object:
                    commands.append('group-object ' + i)
            if address:
                for i in address:
                    commands.append('network-object ' + i)

        elif 'network' in have_group_type:

            if host:
                for i in host:
                    if i not in have_host_ip:
                        if 'object-group network {0}'.format(name) not in commands:
                            commands.append('object-group network {0}'.format(name))
                        commands.append('network-object host ' + i)
            if description:
                if description != have_description:
                    if 'object-group network {0}'.format(name) not in commands:
                        commands.append('object-group network {0}'.format(name))
                    commands.append('description {0}'.format(description))
            if group_object:
                for i in group_object:
                    if i not in have_group_object:
                        if 'object-group network {0}'.format(name) not in commands:
                            commands.append('object-group network {0}'.format(name))
                        commands.append('group-object ' + i)
            if address:
                for i in address:
                    if i not in have_ip_mask:
                        if 'object-group network {0}'.format(name) not in commands:
                            commands.append('object-group network {0}'.format(name))
                        commands.append('network-object ' + i)

    elif 'port-object' in group_type:

        if have_group_type is None and have_protocol != protocol:
            commands.append('object-group service {0} {1}'.format(name, protocol))

            if port_range:
                for i in port_range:
                    commands.append('port-object range ' + i)
            if port_eq:
                for i in port_eq:
                    commands.append('port-object eq ' + i)
            if description:
                commands.append('description {0}'.format(description))

        elif 'port' in have_group_type and have_protocol == protocol:

            if port_range:
                for i in port_range:
                    if i not in have_port_range:
                        if 'object-group service {0} {1}'.format(name, protocol) not in commands:
                            commands.append('object-group service {0} {1}'.format(name, protocol))
                        commands.append('port-object range ' + i)
            if port_eq:
                for i in port_eq:
                    if i not in have_port_eq:
                        if 'object-group service {0} {1}'.format(name, protocol) not in commands:
                            commands.append('object-group service {0} {1}'.format(name, protocol))
                        commands.append('port-object eq ' + i)
            if description:
                if description != have_description:
                    if 'object-group service {0} {1}'.format(name, protocol) not in commands:
                        commands.append('object-group service {0} {1}'.format(name, protocol))
                    commands.append('description {0}'.format(description))

    elif 'service-object' in group_type:

        if have_group_type is None:
            commands.append('object-group service {0}'.format(name))

            if description:
                if have_description is None:
                    commands.append('description {0}'.format(description))
            if service_cfg:
                for i in service_cfg:
                    commands.append('service-object ' + i)

        elif 'service' in have_group_type:

            if description:
                if description != have_description:
                    if 'object-group service {0}'.format(name) not in commands:
                        commands.append('object-group service {0}'.format(name))
                    commands.append('description {0}'.format(description))
            if service_cfg:
                for i in service_cfg:
                    if i not in have_service_cfg:
                        if 'object-group service {0}'.format(name) not in commands:
                            commands.append('object-group service {0}'.format(name))
                        commands.append('service ' + i)

    return commands


def absent(want_dict, have):

    commands = list()

    have_name = have[0].get('have_name')
    have_group_type = have[0].get('have_group_type')
    have_config = have[0].get('have_lines')
    have_description = have[0].get('have_description')
    have_host_ip = have[0].get('have_host_ip')
    have_group_object = have[0].get('have_group_object')
    have_ip_mask = have[0].get('have_ip_mask')
    have_protocol = have[0].get('have_protocol')
    have_port_range = have[0].get('have_port_range')
    have_port_eq = have[0].get('have_port_eq')
    have_service_cfg = have[0].get('have_service_cfg')

    name = want_dict['name']
    group_type = want_dict['group_type']
    protocol = want_dict['protocol']
    description = want_dict['description']
    host = want_dict['host_ip']
    group_object = want_dict['group_object']
    address = want_dict['ip_mask']
    port_range = want_dict['port_range']
    port_eq = want_dict['port_eq']
    service_cfg = want_dict['service_cfg']

    if 'network-object' in group_type:

        if have_group_type is None:
            return commands

        elif 'network' in have_group_type:

            if host:
                for i in host:
                    if i in have_host_ip:
                        if 'object-group network {0}'.format(name) not in commands:
                            commands.append('object-group network {0}'.format(name))
                        commands.append('no network-object host ' + i)
            if description:
                if description == have_description:
                    if 'object-group network {0}'.format(name) not in commands:
                        commands.append('object-group network {0}'.format(name))
                    commands.append('no description {0}'.format(description))
            if group_object:
                for i in group_object:
                    if i in have_group_object:
                        if 'object-group network {0}'.format(name) not in commands:
                            commands.append('object-group network {0}'.format(name))
                        commands.append('no group-object ' + i)
            if address:
                for i in address:
                    if i in have_ip_mask:
                        if 'object-group network {0}'.format(name) not in commands:
                            commands.append('object-group network {0}'.format(name))
                        commands.append('no network-object ' + i)

    elif 'port-object' in group_type:

        if have_group_type is None and have_protocol is None:
            return commands

        elif 'port' in have_group_type and have_protocol == protocol:

            if port_range:
                for i in port_range:
                    if i in have_port_range:
                        if 'object-group service {0} {1}'.format(name, protocol) not in commands:
                            commands.append('object-group service {0} {1}'.format(name, protocol))
                        commands.append('no port-object range ' + i)
            if port_eq:
                for i in port_eq:
                    if i in have_port_eq:
                        if 'object-group service {0} {1}'.format(name, protocol) not in commands:
                            commands.append('object-group service {0} {1}'.format(name, protocol))
                        commands.append('no port-object eq ' + i)
            if description:
                if description == have_description:
                    if 'object-group service {0} {1}'.format(name, protocol) not in commands:
                        commands.append('object-group service {0} {1}'.format(name, protocol))
                    commands.append('no description {0}'.format(description))

    elif 'service-object' in group_type:

        if have_group_type is None:
            return commands

        elif 'service' in have_group_type:
            if description:
                if description == have_description:
                    if 'object-group service {0}'.format(name) not in commands:
                        commands.append('object-group service {0}'.format(name))
                    commands.append('no description {0}'.format(description))
            if service_cfg:
                for i in service_cfg:
                    if i in have_service_cfg:
                        if 'object-group service {0}'.format(name) not in commands:
                            commands.append('object-group service {0}'.format(name))
                        commands.append('no service ' + i)

    return commands


def map_obj_to_commands(want, have, module):

    for w in want:

        want_dict = dict()

        want_dict['name'] = w['name']
        want_dict['group_type'] = w['group_type']
        want_dict['protocol'] = w['protocol']
        want_dict['description'] = w['description']
        want_dict['host_ip'] = w['host_ip']
        want_dict['group_object'] = w['group_object']
        want_dict['ip_mask'] = w['ip_mask']
        want_dict['port_range'] = w['port_range']
        want_dict['port_eq'] = w['port_eq']
        want_dict['service_cfg'] = w['service_cfg']
        state = w['state']

        if state == 'replace':
            return replace(want_dict, have)
        elif state == 'present':
            return present(want_dict, have)
        elif state == 'absent':
            return absent(want_dict, have)


def map_params_to_obj(module):

    obj = list()

    obj.append({
        'name': module.params['name'],
        'group_type': module.params['group_type'],
        'protocol': module.params['protocol'],
        'state': module.params['state'],
        'description': module.params['description'],
        'host_ip': module.params['host_ip'],
        'group_object': module.params['group_object'],
        'port_range': module.params['port_range'],
        'port_eq': module.params['port_eq'],
        'service_cfg': module.params['service_cfg'],
        'ip_mask': module.params['ip_mask']
    })

    return obj


def main():

    argument_spec = dict(
        name=dict(required=True),
        group_type=dict(choices=['network-object', 'service-object', 'port-object'], required=True),
        protocol=dict(choices=['udp', 'tcp', 'tcp-udp']),
        host_ip=dict(type='list'),
        description=dict(),
        group_object=dict(type='list'),
        ip_mask=dict(type='list'),
        port_range=dict(type='list'),
        port_eq=dict(type='list'),
        service_cfg=dict(type='list'),
        state=dict(choices=['present', 'absent', 'replace'], default='present')
    )

    required_if = [('group_type', 'port-object', ['protocol']),
                   ('group_type', 'service-object', ['service_cfg'])]

    module = AnsibleModule(argument_spec=argument_spec,
                           required_if=required_if,
                           supports_check_mode=True)

    result = {'changed': False}

    want = map_params_to_obj(module)
    have = map_config_to_obj(module)
    config_commans = map_obj_to_commands(want, have, module)

    result['commands'] = config_commans

    if config_commans:
        if not module.check_mode:
            load_config(module, config_commans)
        result['changed'] = True

    module.exit_json(**result)


if __name__ == '__main__':
    main()



#!/usr/bin/env python
"""Test to verify action plugins have an associated module to provide documentation."""

import os
import sys


def main():
    """Main entry point."""
    skip = set([
        '__init__',  # action plugin base class, not an actual action plugin
        'net_base',  # base class for other net_* action plugins which have a matching module
        'normal',  # default action plugin for modules without a dedicated action plugin
        'network',  # base class for network action plugins

        # The following action plugins existed without modules to document them before this test was put in place.
        # They should either be removed, have a module added to document them, or have the exception documented here.
        'bigip',
        'bigiq',
        'ce_template',

        # The following action plugins provide base classes for network platform specific modules to support `connection: local`.
        # Once we fully deprecate the use of connection local, the base classes will go away.
        'aireos',
        'aruba',
        'asa',
        'ce',
        'cnos',
        'dellos10',
        'dellos6',
        'dellos9',
        'enos',
        'eos',
        'ios',
        'iosxr',
        'ironware',
        'junos',
        'netconf',
        'nxos',
        'sros',
        'vyos',
    ])

    paths = sys.argv[1:] or sys.stdin.read().splitlines()

    module_names = set()

    for path in paths:
        if not path.startswith('lib/ansible/modules/'):
            continue

        name = os.path.splitext(os.path.basename(path))[0]

        if name != '__init__':
            if name.startswith('_'):
                name = name[1:]

            module_names.add(name)

    unused_skip = set(skip)

    for path in paths:
        if not path.startswith('lib/ansible/plugins/action/'):
            continue

        name = os.path.splitext(os.path.basename(path))[0]

        if name not in module_names:
            if name in skip:
                unused_skip.remove(name)
                continue

            print('%s: action plugin has no matching module to provide documentation' % path)

    for filename in sorted(unused_skip):
        print("%s: remove '%s' from skip list since it does not exist" % ('test/sanity/code-smell/action-plugin-docs.py', filename))


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright: (c) 2018, F5 Networks Inc.
# GNU General Public License v3.0 (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'certified'}

DOCUMENTATION = r'''
---
module: bigip_apm_policy_import
short_description: Manage BIG-IP APM policy or APM access profile imports
description:
   - Manage BIG-IP APM policy or APM access profile imports.
version_added: 2.8
options:
  name:
    description:
      - The name of the APM policy or APM access profile to create or override.
    type: str
    required: True
  type:
    description:
      - Specifies the type of item to export from device.
    type: str
    choices:
      - profile_access
      - access_policy
    default: profile_access
  source:
    description:
      - Full path to a file to be imported into the BIG-IP APM.
    type: path
  force:
    description:
      - When set to C(yes) any existing policy with the same name will be overwritten by the new import.
      - If policy does not exist this setting is ignored.
    default: no
    type: bool
  partition:
    description:
      - Device partition to manage resources on.
    default: Common
notes:
  - Due to ID685681 it is not possible to execute ng_* tools via REST api on v12.x and 13.x, once this is fixed
    this restriction will be removed.
  - Requires BIG-IP >= 14.0.0
extends_documentation_fragment: f5
author:
  - Wojciech Wypior (@wojtek0806)
'''

EXAMPLES = r'''
- name: Import APM profile
  bigip_apm_policy_import:
    name: new_apm_profile
    source: /root/apm_profile.tar.gz
    provider:
      server: lb.mydomain.com
      user: admin
      password: secret
  delegate_to: localhost

- name: Import APM policy
  bigip_apm_policy_import:
    name: new_apm_policy
    source: /root/apm_policy.tar.gz
    type: access_policy
    provider:
      server: lb.mydomain.com
      user: admin
      password: secret
  delegate_to: localhost

- name: Override existing APM policy
  bigip_asm_policy:
    name: new_apm_policy
    source: /root/apm_policy.tar.gz
    force: yes
    provider:
      server: lb.mydomain.com
      user: admin
      password: secret
  delegate_to: localhost
'''

RETURN = r'''
source:
  description: Local path to APM policy file.
  returned: changed
  type: str
  sample: /root/some_policy.tar.gz
name:
  description: Name of the APM policy or APM access profile to be created/overwritten.
  returned: changed
  type: str
  sample: APM_policy_global
type:
  description: Set to specify type of item to export.
  returned: changed
  type: str
  sample: access_policy
force:
  description: Set when overwriting an existing policy or profile.
  returned: changed
  type: bool
  sample: yes
'''

import os
from ansible.module_utils.basic import AnsibleModule
from ansible.module_utils.basic import env_fallback
from distutils.version import LooseVersion

try:
    from library.module_utils.network.f5.bigip import F5RestClient
    from library.module_utils.network.f5.common import F5ModuleError
    from library.module_utils.network.f5.common import AnsibleF5Parameters
    from library.module_utils.network.f5.common import fq_name
    from library.module_utils.network.f5.common import transform_name
    from library.module_utils.network.f5.common import f5_argument_spec
    from library.module_utils.network.f5.icontrol import upload_file
    from library.module_utils.network.f5.icontrol import tmos_version
    from library.module_utils.network.f5.icontrol import module_provisioned
except ImportError:
    from ansible.module_utils.network.f5.bigip import F5RestClient
    from ansible.module_utils.network.f5.common import F5ModuleError
    from ansible.module_utils.network.f5.common import AnsibleF5Parameters
    from ansible.module_utils.network.f5.common import fq_name
    from ansible.module_utils.network.f5.common import transform_name
    from ansible.module_utils.network.f5.common import f5_argument_spec
    from ansible.module_utils.network.f5.icontrol import upload_file
    from ansible.module_utils.network.f5.icontrol import tmos_version
    from ansible.module_utils.network.f5.icontrol import module_provisioned


class Parameters(AnsibleF5Parameters):
    api_map = {

    }

    api_attributes = [

    ]

    returnables = [
        'name',
        'source',
        'type',

    ]

    updatables = [

    ]


class ApiParameters(Parameters):
    pass


class ModuleParameters(Parameters):
    pass


class Changes(Parameters):
    def to_return(self):
        result = {}
        try:
            for returnable in self.returnables:
                result[returnable] = getattr(self, returnable)
            result = self._filter_params(result)
        except Exception:
            pass
        return result


class UsableChanges(Changes):
    pass


class ReportableChanges(Changes):
    pass


class Difference(object):
    def __init__(self, want, have=None):
        self.want = want
        self.have = have

    def compare(self, param):
        try:
            result = getattr(self, param)
            return result
        except AttributeError:
            return self.__default(param)

    def __default(self, param):
        attr1 = getattr(self.want, param)
        try:
            attr2 = getattr(self.have, param)
            if attr1 != attr2:
                return attr1
        except AttributeError:
            return attr1


class ModuleManager(object):
    def __init__(self, *args, **kwargs):
        self.module = kwargs.get('module', None)
        self.client = F5RestClient(**self.module.params)
        self.want = ModuleParameters(params=self.module.params)
        self.changes = UsableChanges()

    def _set_changed_options(self):
        changed = {}
        for key in Parameters.returnables:
            if getattr(self.want, key) is not None:
                changed[key] = getattr(self.want, key)
        if changed:
            self.changes = UsableChanges(params=changed)

    def _announce_deprecations(self, result):
        warnings = result.pop('__warnings', [])
        for warning in warnings:
            self.client.module.deprecate(
                msg=warning['msg'],
                version=warning['version']
            )

    def exec_module(self):
        if not module_provisioned(self.client, 'apm'):
            raise F5ModuleError(
                "APM must be provisioned to use this module."
            )

        if self.version_less_than_14():
            raise F5ModuleError('Due to bug ID685681 it is not possible to use this module on TMOS version below 14.x')

        result = dict()

        changed = self.policy_import()

        reportable = ReportableChanges(params=self.changes.to_return())
        changes = reportable.to_return()
        result.update(**changes)
        result.update(dict(changed=changed))
        self._announce_deprecations(result)
        return result

    def version_less_than_14(self):
        version = tmos_version(self.client)
        if LooseVersion(version) < LooseVersion('14.0.0'):
            return True
        return False

    def policy_import(self):
        self._set_changed_options()
        if self.module.check_mode:
            return True
        if self.exists():
            if self.want.force is False:
                return False

        self.import_file_to_device()
        self.remove_temp_file_from_device()
        return True

    def exists(self):
        if self.want.type == 'access_policy':
            uri = "https://{0}:{1}/mgmt/tm/apm/policy/access-policy/{2}".format(
                self.client.provider['server'],
                self.client.provider['server_port'],
                transform_name(self.want.partition, self.want.name)
            )
        else:
            uri = "https://{0}:{1}/mgmt/tm/apm/profile/access/{2}".format(
                self.client.provider['server'],
                self.client.provider['server_port'],
                transform_name(self.want.partition, self.want.name)
            )
        resp = self.client.api.get(uri)
        try:
            response = resp.json()
        except ValueError:
            return False
        if resp.status == 404 or 'code' in response and response['code'] == 404:
            return False
        return True

    def upload_file_to_device(self, content, name):
        url = 'https://{0}:{1}/mgmt/shared/file-transfer/uploads'.format(
            self.client.provider['server'],
            self.client.provider['server_port']
        )
        try:
            upload_file(self.client, url, content, name)
        except F5ModuleError:
            raise F5ModuleError(
                "Failed to upload the file."
            )

    def import_file_to_device(self):
        name = os.path.split(self.want.source)[1]
        self.upload_file_to_device(self.want.source, name)

        cmd = 'ng_import -s /var/config/rest/downloads/{0} {1} -p {2}'.format(name, self.want.name, self.want.partition)

        uri = "https://{0}:{1}/mgmt/tm/util/bash/".format(
            self.client.provider['server'],
            self.client.provider['server_port'],
        )
        args = dict(
            command='run',
            utilCmdArgs='-c "{0}"'.format(cmd)
        )
        resp = self.client.api.post(uri, json=args)

        try:
            response = resp.json()
            if 'commandResult' in response:
                raise F5ModuleError(response['commandResult'])
        except ValueError as ex:
            raise F5ModuleError(str(ex))

        if 'code' in response and response['code'] == 400:
            if 'message' in response:
                raise F5ModuleError(response['message'])
            else:
                raise F5ModuleError(resp.content)
        return True

    def remove_temp_file_from_device(self):
        name = os.path.split(self.want.source)[1]
        tpath_name = '/var/config/rest/downloads/{0}'.format(name)
        uri = "https://{0}:{1}/mgmt/tm/util/unix-rm/".format(
            self.client.provider['server'],
            self.client.provider['server_port'],
        )
        args = dict(
            command='run',
            utilCmdArgs=tpath_name
        )
        resp = self.client.api.post(uri, json=args)
        try:
            response = resp.json()
        except ValueError as ex:
            raise F5ModuleError(str(ex))
        if 'code' in response and response['code'] == 400:
            if 'message' in response:
                raise F5ModuleError(response['message'])
            else:
                raise F5ModuleError(resp.content)


class ArgumentSpec(object):
    def __init__(self):
        self.supports_check_mode = True
        argument_spec = dict(
            name=dict(
                required=True,
            ),
            source=dict(type='path'),
            force=dict(
                type='bool',
                default='no'
            ),
            type=dict(
                default='profile_access',
                choices=['profile_access', 'access_policy']
            ),
            partition=dict(
                default='Common',
                fallback=(env_fallback, ['F5_PARTITION'])
            )
        )
        self.argument_spec = {}
        self.argument_spec.update(f5_argument_spec)
        self.argument_spec.update(argument_spec)


def main():
    spec = ArgumentSpec()

    module = AnsibleModule(
        argument_spec=spec.argument_spec,
        supports_check_mode=spec.supports_check_mode,
    )

    try:
        mm = ModuleManager(module=module)
        results = mm.exec_module()
        module.exit_json(**results)
    except F5ModuleError as ex:
        module.fail_json(msg=str(ex))


if __name__ == '__main__':
    main()



# Copyright (c) 2018 Remy Leone
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import (absolute_import, division, print_function)

__metaclass__ = type

DOCUMENTATION = '''
    name: netbox
    plugin_type: inventory
    author:
        - Remy Leone (@sieben)
        - Anthony Ruhier (@Anthony25)
        - Nikhil Singh Baliyan (@nikkytub)
    short_description: NetBox inventory source
    description:
        - Get inventory hosts from NetBox
    extends_documentation_fragment:
        - constructed
    options:
        plugin:
            description: token that ensures this is a source file for the 'netbox' plugin.
            required: True
            choices: ['netbox']
        api_endpoint:
            description: Endpoint of the NetBox API
            required: True
            env:
                - name: NETBOX_API
        validate_certs:
            description:
                - Allows connection when SSL certificates are not valid. Set to C(false) when certificates are not trusted.
            default: True
            type: boolean
        config_context:
            description:
                - If True, it adds config-context in host vars.
                - Config-context enables the association of arbitrary data to devices and virtual machines grouped by
                  region, site, role, platform, and/or tenant. Please check official netbox docs for more info.
            default: False
            type: boolean
        token:
            required: True
            description: NetBox token.
            env:
                # in order of precedence
                - name: NETBOX_TOKEN
                - name: NETBOX_API_KEY
        group_by:
            description: Keys used to create groups.
            type: list
            choices:
                - sites
                - tenants
                - racks
                - tags
                - device_roles
                - device_types
                - manufacturers
                - platforms
            default: []
        query_filters:
            description: List of parameters passed to the query string (Multiple values may be separated by commas)
            type: list
            default: []
        timeout:
            description: Timeout for Netbox requests in seconds
            type: int
            default: 60
        compose:
            description: List of custom ansible host vars to create from the device object fetched from NetBox
            default: {}
            type: dict
'''

EXAMPLES = '''
# netbox_inventory.yml file in YAML format
# Example command line: ansible-inventory -v --list -i netbox_inventory.yml

plugin: netbox
api_endpoint: http://localhost:8000
validate_certs: True
config_context: False
group_by:
  - device_roles
query_filters:
  - role: network-edge-router

# Query filters are passed directly as an argument to the fetching queries.
# You can repeat tags in the query string.

query_filters:
  - role: server
  - tag: web
  - tag: production

# See the NetBox documentation at https://netbox.readthedocs.io/en/latest/api/overview/
# the query_filters work as a logical **OR**
#
# Prefix any custom fields with cf_ and pass the field value with the regular NetBox query string

query_filters:
  - cf_foo: bar

# NetBox inventory plugin also supports Constructable semantics
# You can fill your hosts vars using the compose option:

plugin: netbox
compose:
  foo: last_updated
  bar: display_name
  nested_variable: rack.display_name
'''

import json
import uuid
from sys import version as python_version
from threading import Thread
from itertools import chain

from ansible.plugins.inventory import BaseInventoryPlugin, Constructable
from ansible.module_utils.ansible_release import __version__ as ansible_version
from ansible.errors import AnsibleError
from ansible.module_utils._text import to_text
from ansible.module_utils.urls import open_url
from ansible.module_utils.six.moves.urllib.parse import urljoin, urlencode
from ansible.module_utils.compat.ipaddress import ip_interface

ALLOWED_DEVICE_QUERY_PARAMETERS = (
    "asset_tag",
    "cluster_id",
    "device_type_id",
    "has_primary_ip",
    "is_console_server",
    "is_full_depth",
    "is_network_device",
    "is_pdu",
    "mac_address",
    "manufacturer",
    "manufacturer_id",
    "model",
    "name",
    "platform",
    "platform_id",
    "position",
    "rack_group_id",
    "rack_id",
    "role",
    "role_id",
    "serial",
    "site",
    "site_id",
    "status",
    "tag",
    "tenant",
    "tenant_id",
    "virtual_chassis_id",
)


class InventoryModule(BaseInventoryPlugin, Constructable):
    NAME = 'netbox'

    def _fetch_information(self, url):
        response = open_url(url, headers=self.headers, timeout=self.timeout, validate_certs=self.validate_certs)

        try:
            raw_data = to_text(response.read(), errors='surrogate_or_strict')
        except UnicodeError:
            raise AnsibleError("Incorrect encoding of fetched payload from NetBox API.")

        try:
            return json.loads(raw_data)
        except ValueError:
            raise AnsibleError("Incorrect JSON payload: %s" % raw_data)

    def get_resource_list(self, api_url):
        """Retrieves resource list from netbox API.
         Returns:
            A list of all resource from netbox API.
        """
        if not api_url:
            raise AnsibleError("Please check API URL in script configuration file.")

        hosts_list = []
        # Pagination.
        while api_url:
            self.display.v("Fetching: " + api_url)
            # Get hosts list.
            api_output = self._fetch_information(api_url)
            hosts_list += api_output["results"]
            api_url = api_output["next"]

        # Get hosts list.
        return hosts_list

    @property
    def group_extractors(self):
        return {
            "sites": self.extract_site,
            "tenants": self.extract_tenant,
            "racks": self.extract_rack,
            "tags": self.extract_tags,
            "disk": self.extract_disk,
            "memory": self.extract_memory,
            "vcpus": self.extract_vcpus,
            "device_roles": self.extract_device_role,
            "platforms": self.extract_platform,
            "device_types": self.extract_device_type,
            "config_context": self.extract_config_context,
            "manufacturers": self.extract_manufacturer
        }

    def extract_disk(self, host):
        return host.get("disk")

    def extract_vcpus(self, host):
        return host.get("vcpus")

    def extract_memory(self, host):
        return host.get("memory")

    def extract_platform(self, host):
        try:
            return [self.platforms_lookup[host["platform"]["id"]]]
        except Exception:
            return

    def extract_device_type(self, host):
        try:
            return [self.device_types_lookup[host["device_type"]["id"]]]
        except Exception:
            return

    def extract_rack(self, host):
        try:
            return [self.racks_lookup[host["rack"]["id"]]]
        except Exception:
            return

    def extract_site(self, host):
        try:
            return [self.sites_lookup[host["site"]["id"]]]
        except Exception:
            return

    def extract_tenant(self, host):
        try:
            return [self.tenants_lookup[host["tenant"]["id"]]]
        except Exception:
            return

    def extract_device_role(self, host):
        try:
            if 'device_role' in host:
                return [self.device_roles_lookup[host["device_role"]["id"]]]
            elif 'role' in host:
                return [self.device_roles_lookup[host["role"]["id"]]]
        except Exception:
            return

    def extract_config_context(self, host):
        try:
            if self.config_context:
                url = self.api_endpoint + "/api/dcim/devices/" + str(host["id"])
                device_lookup = self._fetch_information(url)
                return [device_lookup["config_context"]]
        except Exception:
            return

    def extract_manufacturer(self, host):
        try:
            return [self.manufacturers_lookup[host["device_type"]["manufacturer"]["id"]]]
        except Exception:
            return

    def extract_primary_ip(self, host):
        try:
            address = host["primary_ip"]["address"]
            return str(ip_interface(address).ip)
        except Exception:
            return

    def extract_primary_ip4(self, host):
        try:
            address = host["primary_ip4"]["address"]
            return str(ip_interface(address).ip)
        except Exception:
            return

    def extract_primary_ip6(self, host):
        try:
            address = host["primary_ip6"]["address"]
            return str(ip_interface(address).ip)
        except Exception:
            return

    def extract_tags(self, host):
        return host["tags"]

    def refresh_platforms_lookup(self):
        url = self.api_endpoint + "/api/dcim/platforms/?limit=0"
        platforms = self.get_resource_list(api_url=url)
        self.platforms_lookup = dict((platform["id"], platform["name"]) for platform in platforms)

    def refresh_sites_lookup(self):
        url = self.api_endpoint + "/api/dcim/sites/?limit=0"
        sites = self.get_resource_list(api_url=url)
        self.sites_lookup = dict((site["id"], site["name"]) for site in sites)

    def refresh_regions_lookup(self):
        url = self.api_endpoint + "/api/dcim/regions/?limit=0"
        regions = self.get_resource_list(api_url=url)
        self.regions_lookup = dict((region["id"], region["name"]) for region in regions)

    def refresh_tenants_lookup(self):
        url = self.api_endpoint + "/api/tenancy/tenants/?limit=0"
        tenants = self.get_resource_list(api_url=url)
        self.tenants_lookup = dict((tenant["id"], tenant["name"]) for tenant in tenants)

    def refresh_racks_lookup(self):
        url = self.api_endpoint + "/api/dcim/racks/?limit=0"
        racks = self.get_resource_list(api_url=url)
        self.racks_lookup = dict((rack["id"], rack["name"]) for rack in racks)

    def refresh_device_roles_lookup(self):
        url = self.api_endpoint + "/api/dcim/device-roles/?limit=0"
        device_roles = self.get_resource_list(api_url=url)
        self.device_roles_lookup = dict((device_role["id"], device_role["name"]) for device_role in device_roles)

    def refresh_device_types_lookup(self):
        url = self.api_endpoint + "/api/dcim/device-types/?limit=0"
        device_types = self.get_resource_list(api_url=url)
        self.device_types_lookup = dict((device_type["id"], device_type["model"]) for device_type in device_types)

    def refresh_manufacturers_lookup(self):
        url = self.api_endpoint + "/api/dcim/manufacturers/?limit=0"
        manufacturers = self.get_resource_list(api_url=url)
        self.manufacturers_lookup = dict((manufacturer["id"], manufacturer["name"]) for manufacturer in manufacturers)

    def refresh_lookups(self):
        lookup_processes = (
            self.refresh_sites_lookup,
            self.refresh_regions_lookup,
            self.refresh_tenants_lookup,
            self.refresh_racks_lookup,
            self.refresh_device_roles_lookup,
            self.refresh_platforms_lookup,
            self.refresh_device_types_lookup,
            self.refresh_manufacturers_lookup,
        )

        thread_list = []
        for p in lookup_processes:
            t = Thread(target=p)
            thread_list.append(t)
            t.start()

        for thread in thread_list:
            thread.join()

    def validate_query_parameters(self, x):
        if not (isinstance(x, dict) and len(x) == 1):
            self.display.warning("Warning query parameters %s not a dict with a single key." % x)
            return

        k = tuple(x.keys())[0]
        v = tuple(x.values())[0]

        if not (k in ALLOWED_DEVICE_QUERY_PARAMETERS or k.startswith("cf_")):
            msg = "Warning: %s not in %s or starting with cf (Custom field)" % (k, ALLOWED_DEVICE_QUERY_PARAMETERS)
            self.display.warning(msg=msg)
            return
        return k, v

    def refresh_url(self):
        query_parameters = [("limit", 0)]
        if self.query_filters:
            query_parameters.extend(filter(lambda x: x,
                                           map(self.validate_query_parameters, self.query_filters)))
        self.device_url = self.api_endpoint + "/api/dcim/devices/?" + urlencode(query_parameters)
        self.virtual_machines_url = self.api_endpoint + "/api/virtualization/virtual-machines/?" + urlencode(query_parameters)

    def fetch_hosts(self):
        return chain(
            self.get_resource_list(self.device_url),
            self.get_resource_list(self.virtual_machines_url),
        )

    def extract_name(self, host):
        # An host in an Ansible inventory requires an hostname.
        # name is an unique but not required attribute for a device in NetBox
        # We default to an UUID for hostname in case the name is not set in NetBox
        return host["name"] or str(uuid.uuid4())

    def add_host_to_groups(self, host, hostname):
        for group in self.group_by:
            sub_groups = self.group_extractors[group](host)

            if not sub_groups:
                continue

            for sub_group in sub_groups:
                group_name = "_".join([group, sub_group])
                self.inventory.add_group(group=group_name)
                self.inventory.add_host(group=group_name, host=hostname)

    def _fill_host_variables(self, host, hostname):
        for attribute, extractor in self.group_extractors.items():
            if not extractor(host):
                continue
            self.inventory.set_variable(hostname, attribute, extractor(host))

        if self.extract_primary_ip(host):
            self.inventory.set_variable(hostname, "ansible_host", self.extract_primary_ip(host=host))

        if self.extract_primary_ip4(host):
            self.inventory.set_variable(hostname, "primary_ip4", self.extract_primary_ip4(host=host))

        if self.extract_primary_ip6(host):
            self.inventory.set_variable(hostname, "primary_ip6", self.extract_primary_ip6(host=host))

    def main(self):
        self.refresh_lookups()
        self.refresh_url()
        hosts_list = self.fetch_hosts()

        for host in hosts_list:
            hostname = self.extract_name(host=host)
            self.inventory.add_host(host=hostname)
            self._fill_host_variables(host=host, hostname=hostname)

            strict = self.get_option("strict")

            # Composed variables
            self._set_composite_vars(self.get_option('compose'), host, hostname, strict=strict)
            # Complex groups based on jinja2 conditionals, hosts that meet the conditional are added to group
            self._set_composite_vars(self.get_option('compose'), host, hostname, strict=strict)

            # Create groups based on variable values and add the corresponding hosts to it
            self._add_host_to_keyed_groups(self.get_option('keyed_groups'), host, hostname, strict=strict)
            self.add_host_to_groups(host=host, hostname=hostname)

    def parse(self, inventory, loader, path, cache=True):
        super(InventoryModule, self).parse(inventory, loader, path)
        self._read_config_data(path=path)

        # Netbox access
        token = self.get_option("token")
        # Handle extra "/" from api_endpoint configuration and trim if necessary, see PR#49943
        self.api_endpoint = self.get_option("api_endpoint").strip('/')
        self.timeout = self.get_option("timeout")
        self.validate_certs = self.get_option("validate_certs")
        self.config_context = self.get_option("config_context")
        self.headers = {
            'Authorization': "Token %s" % token,
            'User-Agent': "ansible %s Python %s" % (ansible_version, python_version.split(' ')[0]),
            'Content-type': 'application/json'
        }

        # Filter and group_by options
        self.group_by = self.get_option("group_by")
        self.query_filters = self.get_option("query_filters")
        self.main()



#
# (c) 2019, Ansible by Red Hat, inc
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
#

from __future__ import (absolute_import, division, print_function)
__metaclass__ = type

from ansible.module_utils.network.frr.providers.cli.config.bgp.process import Provider
from ansible.modules.network.frr import frr_bgp
from .frr_module import TestFrrModule, load_fixture


class TestFrrBgpModule(TestFrrModule):
    module = frr_bgp

    def setUp(self):
        super(TestFrrBgpModule, self).setUp()
        self._bgp_config = load_fixture('frr_bgp_config')

    def test_frr_bgp(self):
        obj = Provider(params=dict(config=dict(bgp_as=64496, router_id='192.0.2.2', networks=None,
                                               address_family=None), operation='merge'))
        commands = obj.render(self._bgp_config)
        self.assertEqual(commands, ['router bgp 64496', 'bgp router-id 192.0.2.2', 'exit'])

    def test_frr_bgp_idempotent(self):
        obj = Provider(params=dict(config=dict(bgp_as=64496, router_id='192.0.2.1', networks=None,
                                               address_family=None), operation='merge'))
        commands = obj.render(self._bgp_config)
        self.assertEqual(commands, [])

    def test_frr_bgp_remove(self):
        obj = Provider(params=dict(config=dict(bgp_as=64496, networks=None,
                                               address_family=None), operation='delete'))
        commands = obj.render(self._bgp_config)
        self.assertEqual(commands, ['no router bgp 64496'])

    def test_frr_bgp_neighbor(self):
        obj = Provider(params=dict(config=dict(bgp_as=64496, neighbors=[dict(neighbor='192.51.100.2', remote_as=64496)],
                                               networks=None, address_family=None),
                                   operation='merge'))
        commands = obj.render(self._bgp_config)
        self.assertEqual(commands, ['router bgp 64496', 'neighbor 192.51.100.2 remote-as 64496', 'exit'])

    def test_frr_bgp_neighbor_idempotent(self):
        obj = Provider(params=dict(config=dict(bgp_as=64496, neighbors=[dict(neighbor='192.51.100.1', remote_as=64496,
                                                                             timers=dict(keepalive=120, holdtime=360))],
                                               networks=None, address_family=None),
                                   operation='merge'))
        commands = obj.render(self._bgp_config)
        self.assertEqual(commands, [])

    def test_frr_bgp_network(self):
        obj = Provider(params=dict(config=dict(bgp_as=64496, networks=[dict(prefix='192.0.2.0', masklen=24, route_map='RMAP_1')],
                                               address_family=None),
                                   operation='merge'))
        commands = obj.render(self._bgp_config)
        self.assertEqual(sorted(commands), sorted(['router bgp 64496', 'network 192.0.2.0/24 route-map RMAP_1', 'exit']))

    def test_frr_bgp_network_idempotent(self):
        obj = Provider(params=dict(config=dict(bgp_as=64496, networks=[dict(prefix='192.0.1.0', masklen=24, route_map='RMAP_1'),
                                                                       dict(prefix='198.51.100.0', masklen=24, route_map='RMAP_2')],
                                               address_family=None),
                                   operation='merge'))
        commands = obj.render(self._bgp_config)
        self.assertEqual(commands, [])

    def test_frr_bgp_address_family_redistribute(self):
        rd_1 = dict(protocol='ospf', id='233', metric=90, route_map=None)

        config = dict(bgp_as=64496, address_family=[dict(afi='ipv4', safi='unicast', redistribute=[rd_1])],
                      networks=None)

        obj = Provider(params=dict(config=config, operation='merge'))

        commands = obj.render(self._bgp_config)
        cmd = ['router bgp 64496', 'address-family ipv4 unicast', 'redistribute ospf 233 metric 90',
               'exit-address-family', 'exit']
        self.assertEqual(sorted(commands), sorted(cmd))

    def test_frr_bgp_address_family_redistribute_idempotent(self):
        rd_1 = dict(protocol='eigrp', metric=10, route_map='RMAP_3', id=None)
        rd_2 = dict(protocol='static', metric=100, id=None, route_map=None)

        config = dict(bgp_as=64496, address_family=[dict(afi='ipv4', safi='unicast', redistribute=[rd_1, rd_2])],
                      networks=None)

        obj = Provider(params=dict(config=config, operation='merge'))

        commands = obj.render(self._bgp_config)
        self.assertEqual(commands, [])

    def test_frr_bgp_address_family_neighbors(self):
        af_nbr_1 = dict(neighbor='192.51.100.1', maximum_prefix=35, activate=True)
        af_nbr_2 = dict(neighbor='192.51.100.3', route_reflector_client=True, activate=True)

        config = dict(bgp_as=64496, address_family=[dict(afi='ipv4', safi='multicast', neighbors=[af_nbr_1, af_nbr_2])],
                      networks=None)

        obj = Provider(params=dict(config=config, operation='merge'))

        commands = obj.render(self._bgp_config)
        cmd = ['router bgp 64496', 'address-family ipv4 multicast', 'neighbor 192.51.100.1 activate',
               'neighbor 192.51.100.1 maximum-prefix 35', 'neighbor 192.51.100.3 activate',
               'neighbor 192.51.100.3 route-reflector-client', 'exit-address-family', 'exit']
        self.assertEqual(sorted(commands), sorted(cmd))

    def test_frr_bgp_address_family_neighbors_idempotent(self):
        af_nbr_1 = dict(neighbor='2.2.2.2', remove_private_as=True, maximum_prefix=100)

        config = dict(bgp_as=64496, address_family=[dict(afi='ipv4', safi='unicast', neighbors=[af_nbr_1])],
                      networks=None)

        obj = Provider(params=dict(config=config, operation='merge'))

        commands = obj.render(self._bgp_config)
        self.assertEqual(commands, [])

    def test_frr_bgp_address_family_networks(self):
        net = dict(prefix='1.0.0.0', masklen=8, route_map='RMAP_1')
        net2 = dict(prefix='192.168.1.0', masklen=24, route_map='RMAP_2')

        config = dict(bgp_as=64496, address_family=[dict(afi='ipv4', safi='multicast', networks=[net, net2])],
                      networks=None)

        obj = Provider(params=dict(config=config, operation='merge'))

        commands = obj.render(self._bgp_config)
        cmd = ['router bgp 64496', 'address-family ipv4 multicast', 'network 1.0.0.0/8 route-map RMAP_1',
               'network 192.168.1.0/24 route-map RMAP_2', 'exit-address-family', 'exit']
        self.assertEqual(sorted(commands), sorted(cmd))

    def test_frr_bgp_address_family_networks_idempotent(self):
        net = dict(prefix='10.0.0.0', masklen=8, route_map='RMAP_1')
        net2 = dict(prefix='20.0.0.0', masklen=8, route_map='RMAP_2')

        config = dict(bgp_as=64496, address_family=[dict(afi='ipv4', safi='multicast', networks=[net, net2])],
                      networks=None)

        obj = Provider(params=dict(config=config, operation='merge'))

        commands = obj.render(self._bgp_config)
        self.assertEqual(commands, [])

    def test_frr_bgp_operation_override(self):
        net_1 = dict(prefix='1.0.0.0', masklen=8, route_map='RMAP_1')
        net_2 = dict(prefix='192.168.1.0', masklen=24, route_map='RMAP_2')
        nbr_1 = dict(neighbor='192.51.100.1', remote_as=64496, advertisement_interval=120)
        nbr_2 = dict(neighbor='192.51.100.3', remote_as=64496, timers=dict(keepalive=300, holdtime=360))
        af_nbr_1 = dict(neighbor='192.51.100.1', maximum_prefix=35)
        af_nbr_2 = dict(neighbor='192.51.100.3', route_reflector_client=True)

        af_1 = dict(afi='ipv4', safi='unicast', neighbors=[af_nbr_1, af_nbr_2])
        af_2 = dict(afi='ipv4', safi='multicast', networks=[net_1, net_2])
        config = dict(bgp_as=64496, neighbors=[nbr_1, nbr_2], address_family=[af_1, af_2], networks=None)

        obj = Provider(params=dict(config=config, operation='override'))
        commands = obj.render(self._bgp_config)

        cmd = ['no router bgp 64496', 'router bgp 64496', 'neighbor 192.51.100.1 remote-as 64496',
               'neighbor 192.51.100.1 advertisement-interval 120', 'neighbor 192.51.100.3 remote-as 64496',
               'neighbor 192.51.100.3 timers 300 360', 'address-family ipv4 unicast',
               'neighbor 192.51.100.1 maximum-prefix 35', 'neighbor 192.51.100.3 route-reflector-client', 'exit-address-family',
               'address-family ipv4 multicast', 'network 1.0.0.0/8 route-map RMAP_1', 'network 192.168.1.0/24 route-map RMAP_2',
               'exit-address-family', 'exit']

        self.assertEqual(sorted(commands), sorted(cmd))

    def test_frr_bgp_operation_replace(self):
        rd = dict(protocol='ospf', id=223, metric=110, route_map=None)
        net = dict(prefix='10.0.0.0', masklen=8, route_map='RMAP_1')
        net2 = dict(prefix='20.0.0.0', masklen=8, route_map='RMAP_2')

        af_1 = dict(afi='ipv4', safi='unicast', redistribute=[rd])
        af_2 = dict(afi='ipv4', safi='multicast', networks=[net, net2])

        config = dict(bgp_as=64496, address_family=[af_1, af_2], networks=None)
        obj = Provider(params=dict(config=config, operation='replace'))
        commands = obj.render(self._bgp_config)

        cmd = ['router bgp 64496', 'address-family ipv4 unicast', 'redistribute ospf 223 metric 110', 'no redistribute eigrp',
               'no redistribute static', 'exit-address-family', 'exit']

        self.assertEqual(sorted(commands), sorted(cmd))

    def test_frr_bgp_operation_replace_with_new_as(self):
        rd = dict(protocol='ospf', id=223, metric=110, route_map=None)

        af_1 = dict(afi='ipv4', safi='unicast', redistribute=[rd])

        config = dict(bgp_as=64497, address_family=[af_1], networks=None)
        obj = Provider(params=dict(config=config, operation='replace'))
        commands = obj.render(self._bgp_config)

        cmd = ['no router bgp 64496', 'router bgp 64497', 'address-family ipv4 unicast', 'redistribute ospf 223 metric 110',
               'exit-address-family', 'exit']

        self.assertEqual(sorted(commands), sorted(cmd))



#!/usr/bin/python

# (c) 2018-2019, NetApp, Inc
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'certified'}


DOCUMENTATION = '''

module: na_ontap_export_policy_rule

short_description: NetApp ONTAP manage export policy rules
extends_documentation_fragment:
    - netapp.na_ontap
version_added: '2.6'
author: NetApp Ansible Team (@carchi8py) <ng-ansibleteam@netapp.com>

description:
- Create or delete or modify export rules in ONTAP

options:
  state:
    description:
    - Whether the specified export policy rule should exist or not.
    required: false
    choices: ['present', 'absent']
    default: present

  name:
    description:
    - The name of the export rule to manage.
    required: True
    aliases:
    - policy_name

  client_match:
    description:
    - List of Client Match host names, IP Addresses, Netgroups, or Domains

  ro_rule:
    description:
    - List of Read only access specifications for the rule
    choices: ['any','none','never','krb5','krb5i','krb5p','ntlm','sys']

  rw_rule:
    description:
    - List of Read Write access specifications for the rule
    choices: ['any','none','never','krb5','krb5i','krb5p','ntlm','sys']

  super_user_security:
    description:
    - List of Read Write access specifications for the rule
    choices: ['any','none','never','krb5','krb5i','krb5p','ntlm','sys']

  allow_suid:
    description:
    - If 'true', NFS server will honor SetUID bits in SETATTR operation. Default value on creation is 'true'
    type: bool

  protocol:
    description:
    - List of Client access protocols.
    - Default value is set to 'any' during create.
    choices: [any,nfs,nfs3,nfs4,cifs,flexcache]

  rule_index:
    description:
    - rule index of the export policy
    - Required for delete and modify
    - If rule_index is not set for a modify, the module will create another rule with desired parameters

  vserver:
    description:
    - Name of the vserver to use.
    required: true

'''

EXAMPLES = """
    - name: Create ExportPolicyRule
      na_ontap_export_policy_rule:
        state: present
        name: default123
        vserver: ci_dev
        client_match: 0.0.0.0/0,1.1.1.0/24
        ro_rule: krb5,krb5i
        rw_rule: any
        protocol: nfs,nfs3
        super_user_security: any
        allow_suid: true
        hostname: "{{ netapp_hostname }}"
        username: "{{ netapp_username }}"
        password: "{{ netapp_password }}"

    - name: Modify ExportPolicyRule
      na_ontap_export_policy_rule:
        state: present
        name: default123
        rule_index: 100
        client_match: 0.0.0.0/0
        ro_rule: ntlm
        rw_rule: any
        protocol: any
        allow_suid: false
        hostname: "{{ netapp_hostname }}"
        username: "{{ netapp_username }}"
        password: "{{ netapp_password }}"

    - name: Delete ExportPolicyRule
      na_ontap_export_policy_rule:
        state: absent
        name: default123
        rule_index: 100
        vserver: ci_dev
        hostname: "{{ netapp_hostname }}"
        username: "{{ netapp_username }}"
        password: "{{ netapp_password }}"

"""

RETURN = """


"""
import traceback

import json
from ansible.module_utils.basic import AnsibleModule
from ansible.module_utils._text import to_native
import ansible.module_utils.netapp as netapp_utils
from ansible.module_utils.netapp_module import NetAppModule


HAS_NETAPP_LIB = netapp_utils.has_netapp_lib()


class NetAppontapExportRule(object):
    ''' object initialize and class methods '''

    def __init__(self):

        self.argument_spec = netapp_utils.na_ontap_host_argument_spec()
        self.argument_spec.update(dict(
            state=dict(required=False, type='str', choices=['present', 'absent'], default='present'),
            name=dict(required=True, type='str', aliases=['policy_name']),
            protocol=dict(required=False,
                          type='list', default=None,
                          choices=['any', 'nfs', 'nfs3', 'nfs4', 'cifs', 'flexcache']),
            client_match=dict(required=False, type='list'),
            ro_rule=dict(required=False,
                         type='list', default=None,
                         choices=['any', 'none', 'never', 'krb5', 'krb5i', 'krb5p', 'ntlm', 'sys']),
            rw_rule=dict(required=False,
                         type='list', default=None,
                         choices=['any', 'none', 'never', 'krb5', 'krb5i', 'krb5p', 'ntlm', 'sys']),
            super_user_security=dict(required=False,
                                     type='list', default=None,
                                     choices=['any', 'none', 'never', 'krb5', 'krb5i', 'krb5p', 'ntlm', 'sys']),
            allow_suid=dict(required=False, type='bool'),
            rule_index=dict(required=False, type='int'),
            vserver=dict(required=True, type='str'),
        ))

        self.module = AnsibleModule(
            argument_spec=self.argument_spec,
            supports_check_mode=True
        )

        self.na_helper = NetAppModule()
        self.parameters = self.na_helper.set_parameters(self.module.params)
        self.set_playbook_zapi_key_map()

        if HAS_NETAPP_LIB is False:
            self.module.fail_json(
                msg="the python NetApp-Lib module is required")
        else:
            self.server = netapp_utils.setup_na_ontap_zapi(
                module=self.module, vserver=self.parameters['vserver'])

    def set_playbook_zapi_key_map(self):
        self.na_helper.zapi_string_keys = {
            'client_match': 'client-match',
            'name': 'policy-name'
        }
        self.na_helper.zapi_list_keys = {
            'protocol': ('protocol', 'access-protocol'),
            'ro_rule': ('ro-rule', 'security-flavor'),
            'rw_rule': ('rw-rule', 'security-flavor'),
            'super_user_security': ('super-user-security', 'security-flavor'),
        }
        self.na_helper.zapi_bool_keys = {
            'allow_suid': 'is-allow-set-uid-enabled'
        }
        self.na_helper.zapi_int_keys = {
            'rule_index': 'rule-index'
        }

    def set_query_parameters(self):
        """
        Return dictionary of query parameters and
        :return:
        """
        query = {
            'policy-name': self.parameters['name'],
            'vserver': self.parameters['vserver']
        }

        if self.parameters.get('rule_index'):
            query['rule-index'] = self.parameters['rule_index']
        else:
            if self.parameters.get('ro_rule'):
                query['ro-rule'] = {'security-flavor': self.parameters['ro_rule']}
            if self.parameters.get('rw_rule'):
                query['rw-rule'] = {'security-flavor': self.parameters['rw_rule']}
            if self.parameters.get('protocol'):
                query['protocol'] = {'security-flavor': self.parameters['protocol']}
            if self.parameters.get('client_match'):
                query['client-match'] = self.parameters['client_match']
        attributes = {
            'query': {
                'export-rule-info': query
            }
        }
        return attributes

    def get_export_policy_rule(self):
        """
        Return details about the export policy rule
        :param:
            name : Name of the export_policy
        :return: Details about the export_policy. None if not found.
        :rtype: dict
        """
        current, result = None, None
        rule_iter = netapp_utils.zapi.NaElement('export-rule-get-iter')
        rule_iter.translate_struct(self.set_query_parameters())
        try:
            result = self.server.invoke_successfully(rule_iter, True)
        except netapp_utils.zapi.NaApiError as error:
            self.module.fail_json(msg='Error getting export policy rule %s: %s'
                                  % (self.parameters['name'], to_native(error)),
                                  exception=traceback.format_exc())

        if result is not None and \
                result.get_child_by_name('num-records') and int(result.get_child_content('num-records')) >= 1:
            current = dict()
            rule_info = result.get_child_by_name('attributes-list').get_child_by_name('export-rule-info')
            for item_key, zapi_key in self.na_helper.zapi_string_keys.items():
                current[item_key] = rule_info.get_child_content(zapi_key)
            for item_key, zapi_key in self.na_helper.zapi_bool_keys.items():
                current[item_key] = self.na_helper.get_value_for_bool(from_zapi=True,
                                                                      value=rule_info[zapi_key])
            for item_key, zapi_key in self.na_helper.zapi_int_keys.items():
                current[item_key] = self.na_helper.get_value_for_int(from_zapi=True,
                                                                     value=rule_info[zapi_key])
            for item_key, zapi_key in self.na_helper.zapi_list_keys.items():
                parent, dummy = zapi_key
                current[item_key] = self.na_helper.get_value_for_list(from_zapi=True,
                                                                      zapi_parent=rule_info.get_child_by_name(parent))
            current['num_records'] = int(result.get_child_content('num-records'))
        return current

    def get_export_policy(self):
        """
        Return details about the export-policy
        :param:
            name : Name of the export-policy

        :return: Details about the export-policy. None if not found.
        :rtype: dict
        """
        export_policy_iter = netapp_utils.zapi.NaElement('export-policy-get-iter')
        attributes = {
            'query': {
                'export-policy-info': {
                    'policy-name': self.parameters['name'],
                    'vserver': self.parameters['vserver']
                }
            }
        }

        export_policy_iter.translate_struct(attributes)
        try:
            result = self.server.invoke_successfully(export_policy_iter, True)
        except netapp_utils.zapi.NaApiError as error:
            self.module.fail_json(msg='Error getting export policy %s: %s'
                                  % (self.parameters['name'], to_native(error)),
                                  exception=traceback.format_exc())

        if result.get_child_by_name('num-records') and int(result.get_child_content('num-records')) == 1:
            return result

        return None

    def add_parameters_for_create_or_modify(self, na_element_object, values):
        """
            Add children node for create or modify NaElement object
            :param na_element_object: modify or create NaElement object
            :param values: dictionary of cron values to be added
            :return: None
        """
        for key in values:
            if key in self.na_helper.zapi_string_keys:
                zapi_key = self.na_helper.zapi_string_keys.get(key)
                na_element_object[zapi_key] = values[key]
            elif key in self.na_helper.zapi_list_keys:
                parent_key, child_key = self.na_helper.zapi_list_keys.get(key)
                na_element_object.add_child_elem(self.na_helper.get_value_for_list(from_zapi=False,
                                                                                   zapi_parent=parent_key,
                                                                                   zapi_child=child_key,
                                                                                   data=values[key]))
            elif key in self.na_helper.zapi_int_keys:
                zapi_key = self.na_helper.zapi_int_keys.get(key)
                na_element_object[zapi_key] = self.na_helper.get_value_for_int(from_zapi=False,
                                                                               value=values[key])
            elif key in self.na_helper.zapi_bool_keys:
                zapi_key = self.na_helper.zapi_bool_keys.get(key)
                na_element_object[zapi_key] = self.na_helper.get_value_for_bool(from_zapi=False,
                                                                                value=values[key])

    def create_export_policy_rule(self):
        """
        create rule for the export policy.
        """
        for key in ['client_match', 'ro_rule', 'rw_rule']:
            if self.parameters.get(key) is None:
                self.module.fail_json(msg='Error: Missing required param for creating export policy rule %s' % key)
        export_rule_create = netapp_utils.zapi.NaElement('export-rule-create')
        self.add_parameters_for_create_or_modify(export_rule_create, self.parameters)
        try:
            self.server.invoke_successfully(export_rule_create, enable_tunneling=True)
        except netapp_utils.zapi.NaApiError as error:
            self.module.fail_json(msg='Error creating export policy rule %s: %s'
                                  % (self.parameters['name'], to_native(error)), exception=traceback.format_exc())

    def create_export_policy(self):
        """
        Creates an export policy
        """
        export_policy_create = netapp_utils.zapi.NaElement.create_node_with_children(
            'export-policy-create', **{'policy-name': self.parameters['name']})
        try:
            self.server.invoke_successfully(export_policy_create,
                                            enable_tunneling=True)
        except netapp_utils.zapi.NaApiError as error:
            self.module.fail_json(msg='Error creating export-policy %s: %s'
                                  % (self.parameters['name'], to_native(error)),
                                  exception=traceback.format_exc())

    def delete_export_policy_rule(self, rule_index):
        """
        delete rule for the export policy.
        """
        if self.parameters.get('rule_index') is None:
            self.parameters['rule_index'] = rule_index
        export_rule_delete = netapp_utils.zapi.NaElement.create_node_with_children(
            'export-rule-destroy', **{'policy-name': self.parameters['name'],
                                      'rule-index': str(rule_index)})

        try:
            self.server.invoke_successfully(export_rule_delete,
                                            enable_tunneling=True)
        except netapp_utils.zapi.NaApiError as error:
            self.module.fail_json(msg='Error deleting export policy rule %s: %s'
                                  % (self.parameters['name'], to_native(error)),
                                  exception=traceback.format_exc())

    def modify_export_policy_rule(self, params):
        '''
        Modify an existing export policy rule
        :param params: dict() of attributes with desired values
        :return: None
        '''
        export_rule_modify = netapp_utils.zapi.NaElement.create_node_with_children(
            'export-rule-modify', **{'policy-name': self.parameters['name'],
                                     'rule-index': str(self.parameters['rule_index'])})
        self.add_parameters_for_create_or_modify(export_rule_modify, params)
        try:
            self.server.invoke_successfully(export_rule_modify, enable_tunneling=True)
        except netapp_utils.zapi.NaApiError as error:
            self.module.fail_json(msg='Error modifying allow_suid %s: %s'
                                  % (self.parameters['allow_suid'], to_native(error)),
                                  exception=traceback.format_exc())

    def autosupport_log(self):
        netapp_utils.ems_log_event("na_ontap_export_policy_rules", self.server)

    def apply(self):
        ''' Apply required action from the play'''
        self.autosupport_log()
        # convert client_match list to comma-separated string
        if self.parameters.get('client_match') is not None:
            self.parameters['client_match'] = ','.join(self.parameters['client_match'])

        current, modify = self.get_export_policy_rule(), None
        action = self.na_helper.get_cd_action(current, self.parameters)
        if action is None and self.parameters['state'] == 'present':
            modify = self.na_helper.get_modified_attributes(current, self.parameters)

        if self.na_helper.changed:
            if self.module.check_mode:
                pass
            else:
                # create export policy (if policy doesn't exist) only when changed=True
                if not self.get_export_policy():
                    self.create_export_policy()
                if action == 'create':
                    self.create_export_policy_rule()
                elif action == 'delete':
                    if current['num_records'] > 1:
                        self.module.fail_json(msg='Multiple export policy rules exist.'
                                                  'Please specify a rule_index to delete')
                    self.delete_export_policy_rule(current['rule_index'])
                elif modify:
                    self.modify_export_policy_rule(modify)
        self.module.exit_json(changed=self.na_helper.changed)


def main():
    ''' Create object and call apply '''
    rule_obj = NetAppontapExportRule()
    rule_obj.apply()


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# (c) 2017, Simon Dodsley (simon@purestorage.com)
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'community'}


DOCUMENTATION = '''
---
module: purefb_bucket
version_added: "2.8"
short_description:  Manage Object Store Buckets on a  Pure Storage FlashBlade.
description:
    - This module managess object store (s3) buckets on Pure Storage FlashBlade.
author: Pure Storage Ansible Team (@sdodsley) <pure-ansible-team@purestorage.com>
options:
  name:
    description:
      - Bucket Name.
    required: true
    type: str
  account:
    description:
      - Object Store Account for Bucket.
    required: true
    type: str
  state:
    description:
      - Create, delete or modifies a bucket.
    required: false
    default: present
    type: str
    choices: [ "present", "absent" ]
  eradicate:
    description:
      - Define whether to eradicate the bucket on delete or leave in trash.
    required: false
    type: bool
    default: false
extends_documentation_fragment:
- purestorage.fb
'''

EXAMPLES = '''
- name: Create new bucket named foo in account bar
  purefb_bucket:
    name: foo
    account: bar
    fb_url: 10.10.10.2
    api_token: T-55a68eb5-c785-4720-a2ca-8b03903bf641

- name: Delete bucket named foo in account bar
  purefb_bucket:
    name: foo
    account: bar
    state: absent
    fb_url: 10.10.10.2
    api_token: T-55a68eb5-c785-4720-a2ca-8b03903bf641

- name: Recover deleted bucket named foo in account bar
  purefb_bucket:
    name: foo
    account: bar
    state: present
    fb_url: 10.10.10.2
    api_token: T-55a68eb5-c785-4720-a2ca-8b03903bf641

- name: Eradicate bucket named foo in account bar
  purefb_bucket:
    name: foo
    account: bar
    state: absent
    eradicate: true
    fb_url: 10.10.10.2
    api_token: T-55a68eb5-c785-4720-a2ca-8b03903bf641
'''

RETURN = '''
'''

HAS_PURITY_FB = True
try:
    from purity_fb import Bucket, Reference
except ImportError:
    HAS_PURITY_FB = False

from ansible.module_utils.basic import AnsibleModule
from ansible.module_utils.pure import get_blade, purefb_argument_spec


MIN_REQUIRED_API_VERSION = '1.5'


def get_s3acc(module, blade):
    """Return Object Store Account or None"""
    s3acc = None
    accts = blade.object_store_accounts.list_object_store_accounts()
    for acct in range(0, len(accts.items)):
        if accts.items[acct].name == module.params['account']:
            s3acc = accts.items[acct]
    return s3acc


def get_bucket(module, blade):
    """Return Bucket or None"""
    s3bucket = None
    buckets = blade.buckets.list_buckets()
    for bucket in range(0, len(buckets.items)):
        if buckets.items[bucket].name == module.params['name']:
            s3bucket = buckets.items[bucket]
    return s3bucket


def create_bucket(module, blade):
    """Create bucket"""
    changed = False
    try:
        attr = Bucket()
        attr.account = Reference(name=module.params['account'])
        blade.buckets.create_buckets(names=[module.params['name']], account=attr)
        changed = True
    except Exception:
        module.fail_json(msg='Object Store Bucket {0}: Creation failed'.format(module.params['name']))
    module.exit_json(changed=changed)


def delete_bucket(module, blade):
    """ Delete Bucket"""
    changed = False
    try:
        blade.buckets.update_buckets(names=[module.params['name']],
                                     destroyed=Bucket(destroyed=True))
        changed = True
        if module.params['eradicate']:
            try:
                blade.buckets.delete_buckets(names=[module.params['name']])
                changed = True
            except Exception:
                module.fail_json(msg='Object Store Bucket {0}: Eradication failed'.format(module.params['name']))
    except Exception:
        module.fail_json(msg='Object Store Bucket {0}: Deletion failed'.format(module.params['name']))
    module.exit_json(changed=changed)


def recover_bucket(module, blade):
    """ Recover Bucket"""
    changed = False
    try:
        blade.buckets.update_buckets(names=[module.params['name']],
                                     destroyed=Bucket(destroyed=False))
        changed = True
    except Exception:
        module.fail_json(msg='Object Store Bucket {0}: Recovery failed'.format(module.params['name']))
    module.exit_json(changed=changed)


def eradicate_bucket(module, blade):
    """ Eradicate Bucket"""
    changed = False
    try:
        blade.buckets.delete_buckets(names=[module.params['name']])
        changed = True
    except Exception:
        module.fail_json(msg='Object Store Bucket {0}: Eradication failed'.format(module.params['name']))
    module.exit_json(changed=changed)


def main():
    argument_spec = purefb_argument_spec()
    argument_spec.update(
        dict(
            name=dict(required=True),
            account=dict(required=True),
            eradicate=dict(default='false', type='bool'),
            state=dict(default='present', choices=['present', 'absent']),
        )
    )

    module = AnsibleModule(argument_spec)

    if not HAS_PURITY_FB:
        module.fail_json(msg='purity_fb sdk is required for this module')

    state = module.params['state']
    blade = get_blade(module)
    api_version = blade.api_version.list_versions().versions
    if MIN_REQUIRED_API_VERSION not in api_version:
        module.fail_json(msg="Purity//FB must be upgraded to support this module.")
    bucket = get_bucket(module, blade)
    if not get_s3acc(module, blade):
        module.fail_json(msg="Object Store Account {0} does not exist.".format(module.params['account']))

    if state == 'present' and not bucket:
        create_bucket(module, blade)
    elif state == 'present' and bucket and bucket.destroyed:
        recover_bucket(module, blade)
    elif state == 'absent' and bucket and not bucket.destroyed:
        delete_bucket(module, blade)
    elif state == 'absent' and bucket and bucket.destroyed and module.params['eradicate']:
        eradicate_bucket(module, blade)
    elif state == 'absent' and not bucket:
        module.exit_json(changed=False)

    module.exit_json(changed=False)


if __name__ == '__main__':
    main()



#!/usr/bin/python
# Copyright: Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'network'}


DOCUMENTATION = """
---
module: restconf_get
version_added: "2.8"
author: "Ganesh Nalawade (@ganeshrn)"
short_description: Fetch configuration/state data from RESTCONF enabled devices.
description:
    - RESTCONF is a standard mechanisms to allow web applications to access the
      configuration data and state data developed and standardized by
      the IETF. It is documented in RFC 8040.
    - This module allows the user to fetch configuration and state data from RESTCONF
      enabled devices.
options:
  path:
    description:
      - URI being used to execute API calls.
    required: true
  content:
    description:
      - The C(content) is a query parameter that controls how descendant nodes of the
        requested data nodes in C(path) will be processed in the reply. If value is
        I(config) return only configuration descendant data nodes of value in C(path).
        If value is I(nonconfig) return only non-configuration descendant data nodes
        of value in C(path). If value is I(all) return all descendant data nodes of
        value in C(path)
    required: false
    choices: ['config', 'nonconfig', 'all']
  output:
    description:
      - The output of response received.
    required: false
    default: json
    choices: ['json', 'xml']
"""

EXAMPLES = """
- name: get l3vpn services
  restconf_get:
      path: /config/ietf-l3vpn-svc:l3vpn-svc/vpn-services
"""

RETURN = """
response:
  description: A dictionary representing a JSON-formatted response
  returned: when the device response is valid JSON
  type: dict
  sample: |
        {
            "vpn-services": {
                "vpn-service": [
                    {
                        "customer-name": "red",
                        "vpn-id": "blue_vpn1",
                        "vpn-service-topology": "ietf-l3vpn-svc:any-to-any"
                    }
                ]
            }
        }

"""

from ansible.module_utils._text import to_text
from ansible.module_utils.basic import AnsibleModule
from ansible.module_utils.connection import ConnectionError
from ansible.module_utils.network.restconf import restconf


def main():
    """entry point for module execution
    """
    argument_spec = dict(
        path=dict(required=True),
        content=dict(choices=['config', 'nonconfig', 'all']),
        output=dict(choices=['json', 'xml'], default='json'),
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True
    )

    result = {'changed': False}

    try:
        response = restconf.get(module, **module.params)
    except ConnectionError as exc:
        module.fail_json(msg=to_text(exc), code=exc.code)

    result.update({
        'response': response,
    })

    module.exit_json(**result)


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-

# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)


class ModuleDocFragment(object):
    # Standard F5 documentation fragment
    DOCUMENTATION = r'''
options:
  password:
    description:
      - The password for the user account used to connect to the BIG-IP.
      - You may omit this option by setting the environment variable C(F5_PASSWORD).
    type: str
    required: true
    aliases: [ pass, pwd ]
  server:
    description:
      - The BIG-IP host.
      - You may omit this option by setting the environment variable C(F5_SERVER).
    type: str
    required: true
  server_port:
    description:
      - The BIG-IP server port.
      - You may omit this option by setting the environment variable C(F5_SERVER_PORT).
    type: int
    default: 443
    version_added: '2.2'
  user:
    description:
      - The username to connect to the BIG-IP with. This user must have
        administrative privileges on the device.
      - You may omit this option by setting the environment variable C(F5_USER).
    type: str
    required: true
  validate_certs:
    description:
      - If C(no), SSL certificates are not validated. Use this only
        on personally controlled sites using self-signed certificates.
      - You may omit this option by setting the environment variable
        C(F5_VALIDATE_CERTS).
    type: bool
    default: yes
    version_added: '2.0'
  provider:
    description:
      - A dict object containing connection details.
    type: dict
    version_added: '2.5'
    suboptions:
      password:
        description:
          - The password for the user account used to connect to the BIG-IP.
          - You may omit this option by setting the environment variable C(F5_PASSWORD).
        type: str
        required: true
        aliases: [ pass, pwd ]
      server:
        description:
          - The BIG-IP host.
          - You may omit this option by setting the environment variable C(F5_SERVER).
        type: str
        required: true
      server_port:
        description:
          - The BIG-IP server port.
          - You may omit this option by setting the environment variable C(F5_SERVER_PORT).
        type: int
        default: 443
      user:
        description:
          - The username to connect to the BIG-IP with. This user must have
            administrative privileges on the device.
          - You may omit this option by setting the environment variable C(F5_USER).
        type: str
        required: true
      validate_certs:
        description:
          - If C(no), SSL certificates are not validated. Use this only
            on personally controlled sites using self-signed certificates.
          - You may omit this option by setting the environment variable C(F5_VALIDATE_CERTS).
        type: bool
        default: yes
      timeout:
        description:
          - Specifies the timeout in seconds for communicating with the network device
            for either connecting or sending commands.  If the timeout is
            exceeded before the operation is completed, the module will error.
        type: int
        default: 10
      ssh_keyfile:
        description:
          - Specifies the SSH keyfile to use to authenticate the connection to
            the remote device.  This argument is only used for I(cli) transports.
          - You may omit this option by setting the environment variable C(ANSIBLE_NET_SSH_KEYFILE).
        type: path
      transport:
        description:
          - Configures the transport connection to use when connecting to the
            remote device.
        type: str
        choices: [ cli, rest ]
        default: rest
notes:
  - For more information on using Ansible to manage F5 Networks devices see U(https://www.ansible.com/integrations/networks/f5).
  - Requires BIG-IP software version >= 12.
  - The F5 modules only manipulate the running configuration of the F5 product. To ensure that BIG-IP
    specific configuration persists to disk, be sure to include at least one task that uses the
    M(bigip_config) module to save the running configuration. Refer to the module's documentation for
    the correct usage of the module to save your running configuration.
'''
